#include "isf_bul.h"

/* Functions to read and write IMS station data_type lines. */
/* Start and stop dates need doing properly if want to use them. */


/*  Writes a station line.

    Returns 0 on a successful write.  
    Returns 20 if any of the variables passed are unsuitable for writing.
    On error writes a diagnostic to isf_bulletin_error.
*/
int write_sta(FILE *fp, char *net, char *sta, char *type, float lat,
				float lon, char *coord, float elev, char *start, char *stop)
{

    /* Chars 1-9: network code - can be null. Char 10: space */
    if (net[0] == '\0' ){
        fprintf (fp,"%*s ",ISF_NET_LEN,"");
    }
    else {
        if (strlen(net) > ISF_NET_LEN ){
            sprintf (isf_bulletin_error,"net too long: %s\n",net); 
            return 20;
        }
        if (check_whole(net)){
            sprintf (isf_bulletin_error,"bad net: %s\n",net); 
            return 20;
        }
        fprintf (fp,"%-*s ",ISF_NET_LEN,net);
    }


    /* Chars 11-15: station code. Char 16: space  */
    if (sta[0] == '\0' ){
        sprintf (isf_bulletin_error,"missing sta\n"); 
        return 20;
    }
    if (strlen(sta) > ISF_STA_LEN ){
        sprintf (isf_bulletin_error,"sta too long: %s\n",sta); 
        return 20;
    }
    if (check_whole(sta)){
        sprintf (isf_bulletin_error,"bad sta: %s\n",sta); 
        return 20;
    }
    fprintf (fp,"%-*s ",ISF_STA_LEN,sta);


    /* Chars 17-20: station type - can be null. Char 21: space */
    if (type[0] == '\0' ){
        fprintf (fp,"%*s ",4,"");
    }
    else {
        if (strlen(type) > 4 ){
            sprintf (isf_bulletin_error,"type too long: %s\n",type); 
            return 20;
        }
        if (check_whole(type)){
            sprintf (isf_bulletin_error,"bad type: %s\n",type); 
            return 20;
        }
        fprintf (fp,"%-*s ",4,type);
    }


    /* Chars 22-30: lattitude. Char 31: space. */
    if (is_null(lat)){
        sprintf (isf_bulletin_error,"missing latitude\n"); 
        return 20;
    }
    if ( lat <= -90 || lat > 90 ){
        sprintf (isf_bulletin_error,"bad lat: %f\n",lat); 
        return 20;
    }
    fprintf (fp,"%9.5f ",lat);

    /* Chars 32-41: longitude. Char 42: space. */
    if (is_null(lon)){
        sprintf (isf_bulletin_error,"missing longitude\n"); 
        return 20;
    }
    if ( lon < -180 || lon > 180 ){
        sprintf (isf_bulletin_error,"bad lon: %f\n",lon); 
        return 20;
    }
    fprintf (fp,"%10.5f ",lon);

    /* Chars 43-54: coordinate system - can be null. */
    if (!coord[0]){
       fprintf (fp,"%*s",12,"");
    }
    else {
        if (strlen(coord) > 12 ){
            sprintf (isf_bulletin_error,"coord too long: %s\n",coord); 
            return 20;
        }
        if (check_whole(coord)){
            sprintf (isf_bulletin_error,"bad coord: %s\n",coord); 
            return 20;
        }
        fprintf (fp,"%-*s",12,coord);
    }

	/* Char 55: either space or - from elevation.			*/
	/* Chars 56-60 elevation - can be null. Char 61 space.	*/
    if (is_null(elev)){
        fprintf (fp,"      ");
    }
    else{
        if ( elev < -9 || elev > 9 ){
            sprintf (isf_bulletin_error,"bad elev: %f\n",elev); 
            return 20;
        }
        fprintf (fp,"% 5.3f ",elev);
    }

    /* Chars 62-71: start date - can be null. Char 72: space */
    if (start[0] == '\0' ){
        fprintf (fp,"%*s ",10,"");
    }
    else {
        if (strlen(start) > 10 ){
            sprintf (isf_bulletin_error,"start too long: %s\n",start); 
            return 20;
        }
        if (check_whole(start)){
            sprintf (isf_bulletin_error,"bad start: %s\n",start); 
            return 20;
        }
        fprintf (fp,"%-*s ",10,start);
    }

    /* Chars 73-82: stop date - can be null. */
    if (stop[0] == '\0' ){
        fprintf (fp,"%*s\n",10,"");
    }
    else {
        if (strlen(stop) > 10 ){
            sprintf (isf_bulletin_error,"stop too long: %s\n",stop); 
            return 20;
        }
        if (check_whole(stop)){
            sprintf (isf_bulletin_error,"bad stop: %s\n",stop); 
            return 20;
        }
        fprintf (fp,"%-*s\n",10,stop);
    }
    return 0;
}


/*  Parses a line asuming it to be a station line.
    Values are asigned to variables, the pointers to which have been sent
    as arguments. If an optional parameter is not given then the
    corresponding variable will have ISF_NULL assigned to it.

    Returns 0 if the line is a properly formatted origin line.
    Returns 20 and writes a diagnostic to isf_bulletin_error on error.
*/
int read_sta(char *line, char *net, char *sta, char *type, float *lat,
				float *lon, char *coord, float *elev, char *start, char *stop)
{
    char substr[ISF_LINE_LEN];

    /* Chars 1-9: network code. */
     if (partline(net,line,0,9)){
        if (check_whole(net)){
            sprintf (isf_bulletin_error,"bad net: %s",line); 
            return 20;
        }
    }
    else{
        strcpy(net,"");
    }

    /* Char 10: must be a space. */
    if (line[9] != ' ' ){
        sprintf (isf_bulletin_error,"bad format, char 10: %s",line); 
        return 20;
    }

    /* Chars 11-15: station code. */
    if (!partline(sta,line,10,5)){
        sprintf (isf_bulletin_error,"missing sta: %s",line); 
        return 20;
    }
    if (check_whole(sta)){
        sprintf (isf_bulletin_error,"bad sta: %s",line); 
        return 20;
    }

    /* Char 16: must be a space. */
    if (line[15] != ' ' ){
        sprintf (isf_bulletin_error,"bad format, char 16: %s",line); 
        return 20;
    }

    /* Chars 17-20: network code. */
     if (partline(type,line,16,4)){
        if (check_whole(type)){
            sprintf (isf_bulletin_error,"bad type: %s",line); 
            return 20;
        }
    }
    else{
        strcpy(type,"");
    }

    /* Char 21: space. */
    if (line[20] != ' '){
        sprintf (isf_bulletin_error,"bad format, char 21: %s",line); 
        return 20;
    }

    /* Chars 22-30: lattitude - must be float. */
    if (!partline(substr,line,21,9)){
        sprintf (isf_bulletin_error,"missing lattitude: %s",line); 
        return 20;
    }
    if (check_float(substr)){
        sprintf (isf_bulletin_error,"bad lattitude: %s",line); 
        return 20;
    }
    *lat = (float)atof(substr);

    /* Char 31: space. */
    if (line[30] != ' '){
        sprintf (isf_bulletin_error,"bad format, char 31: %s",line); 
        return 20;
    }

   /* Chars 32-41: longitude - must be float. */
    if (!partline(substr,line,31,10)){
        sprintf (isf_bulletin_error,"missing longitude: %s",line); 
        return 20;
    }
    if (check_float(substr)){
        sprintf (isf_bulletin_error,"bad longitude: %s",line); 
        return 20;
    }
    *lon = (float)atof(substr);

    /* Char 42: space. */
    if (line[41] != ' '){
        sprintf (isf_bulletin_error,"bad format, char 42: %s",line); 
        return 20;
    }

    /* Chars 43-54: network code. */
     if (partline(coord,line,42,12)){
        if (check_whole(coord)){
            sprintf (isf_bulletin_error,"bad coord: %s",line); 
            return 20;
        }
    }
    else{
        strcpy(coord,"");
    }

    /* Chars 56-60: elevation. */
    if (partline(substr,line,54,6)){
       if (check_float(substr)){
            sprintf (isf_bulletin_error,"bad elev: %s",line); 
            return 20;
        }
        *elev = (float)atof(substr);
    }
    else {
        *elev = ISF_NULL;
    }

    /* Char 61: space. */
    if (line[60] != ' '){
        sprintf (isf_bulletin_error,"bad format, char 61: %s",line); 
        return 20;
    }

    /* Chars 62-71: start of opperation. */
     if (partline(start,line,61,10)){
        if (check_whole(start)){
            sprintf (isf_bulletin_error,"bad start: %s",line); 
            return 20;
        }
    }
    else{
        strcpy(start,"");
    }

    /* Char 72: space. */
    if (line[71] != ' '){
        sprintf (isf_bulletin_error,"bad format, char 72: %s",line); 
        return 20;
    }

    /* Chars 73-82: start of opperation. */
     if (partline(stop,line,72,10)){
        if (check_whole(stop)){
            sprintf (isf_bulletin_error,"bad stop: %s",line); 
            return 20;
        }
    }
    else{
        strcpy(stop,"");
    }

    /* Check for extra stuff after char 82. */
    if (partline(substr,line,82,0)){
        sprintf (isf_bulletin_error,"extra characters at end: %s",line); 
        return 20;
    } 
    return 0;
}
