#include "location.h"

/*#DOC  Title:																*/
/*#DOC    buland_weight														*/
/*#DOC  Desc:																*/
/*#DOC    Fills 'alpha' in solution structure and 'weight' for each phase.	*/
/*#DOC  Input Arguments:													*/
/*#DOC    sp - pointer to structure containing current solution.			*/
/*#DOC    p[]  - array structures containing phases.						*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/

/* Weighting as implemented in reviser.								*/
/* Not in fact full Jeffreys weighting as sigval remains constant throughout. */

/* Designed to be run after Huber-t weighting has converged - see	*/
/* comment in huber_weighting.c.									*/

int buland_weight (struct sol_rec *sp, struct pha_rec p[])
{
	
	extern int min_phases;				/* From config file */
	extern double init_max_resid;		/* From config file */
	extern double max_resid;			/* From config file */
	extern double mu;					/* From config file */
	extern double sigma_start;			/* From config file */

	int i,ndef;
	double total_weight;
	double sigma_12;
	double sigval;
	double spread;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	sigma_12 = 12*sigma_start;
	sigval   = 1/(2*sigma_start*sigma_start);	/* Robust sigma from R.Buland.*/

	/* First iteration initialise alpha as average residual.		*/
	/* Then calculate initial weights.								*/
	/* Afterwards, calculate weights again as usual.				*/
	if (sp->iteration == 0){
		sp->alpha=0;
		ndef=0;

		for (i=0; i<sp->numphas; i++){
			if (p[i].weight_factor > 0 && fabs(p[i].resid) < init_max_resid){
				sp->alpha += p[i].resid;
				ndef++;
			}
			if (diagnostic){
				fprintf(logfp,"buland_weight: init:%s %s ",p[i].sta,p[i].phase);
				fprintf(logfp,"%f %f\n",p[i].weight_factor,p[i].resid);
			}
		}
		if (ndef < min_phases ){
			fprintf(logfp,"TOO FEW PHASES buland_weight init: ndef=%d\n",ndef);
			strcpy(errstr,"");
			return 1;
		}
		sp->alpha /= ndef;
		sp->prev_alpha = sp->alpha;

		if (diagnostic)
			fprintf(logfp,"buland_weight: init alpha=%f n=%d\n",sp->alpha,ndef);

		for (i=0; i<sp->numphas; i++){
			if (fabs(p[i].resid) < max_resid){
				spread = fabs(p[i].resid - sp->alpha);
				if (spread <= sigma_12)
					p[i].weight = 1/(1+mu*exp(spread*spread*sigval));
				else
					p[i].weight = 0;

				p[i].weight *= p[i].weight_factor;
			}
			else
				p[i].weight = 0;
		}
	}

	/* First calculate alpha. */
	sp->alpha=0;
	total_weight=0;
	ndef=0;
	for (i=0; i<sp->numphas; i++){
		if (p[i].weight_factor > 0 && fabs(p[i].resid) < max_resid){
			sp->alpha += p[i].resid*p[i].weight;
			total_weight += p[i].weight;
			ndef++;
		}
		if (diagnostic){
			fprintf(logfp,"buland_weight: %s %s ",p[i].sta,p[i].phase);
			fprintf(logfp,"%f %f ",p[i].weight_factor,p[i].weight);
			fprintf(logfp,"%f\n",p[i].resid);
		}
	}
	if (ndef < min_phases ){
		fprintf(logfp,"TOO FEW PHASES buland_weight: ndef=%d\n",ndef);
		strcpy(errstr,"");
		return 1;
	}
	sp->alpha /= total_weight;

	if (diagnostic){
		fprintf(logfp,"buland_weight: alpha=%f ",sp->alpha);
		fprintf(logfp,"total_weight=%f\n",total_weight);
	}

	/* Then calculate weights. */
	for (i=0; i<sp->numphas; i++){
		if (fabs(p[i].resid) < max_resid){
			spread = fabs(p[i].resid - sp->alpha);
			if (spread <= sigma_12)
				p[i].weight = 1/(1+mu*exp(spread*spread*sigval));
			else
				p[i].weight = 0;

			p[i].weight *= p[i].weight_factor;
		}
		else
			p[i].weight = 0;
	}
	return 0;
}
