#include "location.h"

/*#DOC  Title:																*/
/*#DOC    calc_error														*/
/*#DOC  Desc:																*/
/*#DOC    Calculates 'nass', 'ndef', 'nsta', 'ndefsta', 'sdobs', 'mindist',	*/
/*#DOC    'maxdist', 'azimgap', 'error[]','majax', 'minax', and 'theta'		*/
/*#DOC  Input Arguments:													*/
/*#DOC    sp  - pointer to structure containing solution.					*/
/*#DOC    p[] - array of pointers to structures containing phases.			*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/
/*#DOC  Uses:																*/
/*#DOC    get_fact() and get_fact2() from this file.						*/


/* Counts phases/stations to get ndef etc. for final solution:			*/
/* nsta is all stations except those that have had phases purged.		*/
/* (Stations with purged phases will be removed from this event.)		*/
/* nass is all phases for all nsta stations.							*/
/* ndef is number of phases with a weight_factor > 0.					*/
/* ndefsta is count of stations with at least one defining phase.		*/

/* Calculates weighted standard deviation of residuals using the		*/
/*  current value of sigma. (Only actualy sd if assume mean resid=0).	*/

/* Interprets the covarience matrix to get standard errors and an		*/
/* error ellipse. Assumes that dtdd is in sec/degree.					*/

int calc_error (struct sol_rec *sp, struct pha_rec p[])
{

	double get_fact (int num, int nunk);	/* Local subroutine */
	double get_fact2(int num, int nunk);	/* Local subroutine */

	double *esaz;
	double var_lat,var_lon,lat_lon;
	double scale_fact,km_per_deg;
	double b,c,eigen1,eigen2;
	char prev_net[NETLEN],prev_sta[STALEN];
	char prev_def_net[NETLEN],prev_def_sta[STALEN];
	int pha_for_sta, purge_for_sta, weight_for_sta;
	int i;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* Assign memory for array of azimuths. */
	if ((esaz = (double*) calloc (sp->numphas, sizeof(double))) == NULL){
		sprintf(errstr,"calc_error: Memory problem.");
		return 1;
	}

	/* Count associated/defining phases/stations. 					*/
	/* Rely on phases being ordered by station in array.	 		*/
	/* Exclude a station from nsta if one or more phases purged		*/
	/* and no other phases for it have a weight.					*/
	/* nass includes all or none of the phases for a given station.	*/
	/* At same time get mindist and maxdist and fill an array of	*/
	/* azimuths for the calculation of azimgap.						*/
	sp->ndef = sp->nass = sp->nsta = sp->ndefsta = 0;
	pha_for_sta = purge_for_sta = weight_for_sta = 0;
	sp->mindist = 360;
	sp->maxdist = 0;
	for (i=0;i<sp->numphas;i++){

		/* New station. */
		if (strcmp(p[i].sta,prev_sta) || strcmp(p[i].net,prev_net)){

			/* Add numbers for previous station. */
			if (pha_for_sta > 0 && (purge_for_sta==0 || weight_for_sta > 0)){
				sp->nsta++;
				sp->nass+=pha_for_sta;
			}
			pha_for_sta = purge_for_sta = weight_for_sta = 0;
		}
		pha_for_sta++;

		/* Defining phases/stations. */
		if (p[i].weight_factor > 0 && p[i].resid != NULLVAL){

			/* New defining station. */
			if (strcmp(p[i].sta,prev_def_sta) || strcmp(p[i].net,prev_def_net)){
				if (p[i].delta < sp->mindist)
					sp->mindist = p[i].delta;
				if (p[i].delta > sp->maxdist)
					sp->maxdist = p[i].delta;

				esaz[sp->ndefsta++] = p[i].esaz;
				strcpy(prev_def_sta,p[i].sta);
				strcpy(prev_def_net,p[i].net);
			}
			sp->ndef++;
			weight_for_sta++;
		}

		if (p[i].purged)
			purge_for_sta++;

		strcpy(prev_sta,p[i].sta);
		strcpy(prev_net,p[i].net);
	}
	/* Add numbers for last station. */
	if (pha_for_sta > 0 && (purge_for_sta==0 || weight_for_sta > 0)){
		sp->nsta++;
		sp->nass+=pha_for_sta;
	}

	/* Weighted standard deviation - can use final value of sigma. */
	if (sp->ndef){
		sp->sdobs =  sp->sigma;
		sp->sdobs *= sqrt(sp->ndef/(sp->ndef - sp->number_of_unknowns));
	}
	else {
		sp->sdobs   = NULLVAL;
		sp->mindist = NULLVAL;
		sp->maxdist = NULLVAL;
	}

	/* Calculate azimuthal gap using local array of azimuths filled above. */
	if (sp->ndef){
		dsort(esaz,0,sp->ndefsta-1);
		sp->azimgap = esaz[0] + 360 - esaz[sp->ndefsta-1];

		for(i=0; i < sp->ndefsta-1; i++){
			if( (esaz[i+1]-esaz[i]) > sp->azimgap)
				sp->azimgap = esaz[i+1]-esaz[i];
		}
	}
	else {
		sp->azimgap = NULLVAL;
		sp->ndef = NULLVAL;
		sp->ndefsta = NULLVAL;
	}
	free(esaz);

	if (diagnostic){
		fprintf(logfp,"calc_error: nsta=%d nass=%d ",sp->nsta,sp->nass);
		fprintf(logfp,"ndefsta=%d ndef=%d ",sp->ndefsta,sp->ndef);
		fprintf(logfp,"gap=%f \n",sp->azimgap);
	}

	/* Calculate standard errors. */

	/* Calculate scaling factor for errors using function below. */
	if (diagnostic)
		fprintf(logfp,"calc_error: calling get_fact\n");
	scale_fact = get_fact(sp->numphas,sp->number_of_unknowns);

	/* Enter errors in solution structure. */
	for (i=1; i<=sp->number_of_unknowns; i++)
		sp->error[i] = scale_fact*sqrt(sp->covar[i][i]);
	for (i=sp->number_of_unknowns+1; i<5; i++)
		sp->error[i] = NULLVAL;

	/* No error ellipse for fixed location solution */
	if (sp->number_of_unknowns < 3){
		sp->majax    = NULLVAL;
		sp->minax    = NULLVAL;
		sp->theta    = NULLVAL;
 		return 0;
	}

	/* Express longitude error in degrees for appropriate latitude */
	sp->error[3] /= cos(sp->lat*TORAD);

	/* Calculate error ellipse. */

	/* Get partial derivatives from matrix and convert into km. */
	/* Terms for longitude are already in degrees of equatorial equivalent. */
	km_per_deg = TORAD * (EARTH_RADIUS - sp->depth);
	var_lat = km_per_deg * km_per_deg * sp->covar[2][2];
	lat_lon = km_per_deg * km_per_deg * sp->covar[2][3];
	var_lon = km_per_deg * km_per_deg * sp->covar[3][3];

	/* Solve quadratic to get the eigenvalues of the matrix:		*/
	/*		 x = (-b +/- sqrt(b^2-4ac)) / 2a  where a=1				*/
	b = var_lat+var_lon;
	c = var_lat*var_lon - lat_lon*lat_lon;

	eigen1 = 0.5 * (b + sqrt(b*b - 4*c));
	eigen2 = 0.5 * (b - sqrt(b*b - 4*c));

	/* Again need scaling factor, but a differnet one this time. */
	if (diagnostic)
		fprintf(logfp,"calc_error: calling get_fact2\n");
	scale_fact = get_fact2(sp->numphas,sp->number_of_unknowns);

	/* Calculate ellipse parameters. */
	sp->majax = sqrt(eigen1)*scale_fact;
	sp->minax = sqrt(eigen2)*scale_fact;
	sp->theta = (atan(2*lat_lon/(var_lat-var_lon)))/TORAD;

	if (sp->theta < 0)
		sp->theta += 180;

	/* Print out avergage ellipse axis for cf reviser. */
	if (diagnostic)
		printf ("calc_error: position %f\n", sqrt(sp->majax*sp->minax));

	return 0;
}

/* Subroutine to get error scaling factor.  Only used by calc_error. */
/* Comments below come from reviser function fctrn.f:			*/
/*		N is num, M is nunk										*/
/*  
c   Function fctrn returns a semi-empirical scaling factor which when
c   multiplied into the square roots of the diagonal elements of the
c   correlation matrix, yield the best current estimates of the 90%
c   marginal confidence intervals.  N is the number of observations
c   and M is the number of parameters being found (either 3 for a held
c   depth or 4 for a free depth solution).  The factor is
c   asymptotically sigma times beta times the .05 percentage point of
c   the normal distribution.  Sigma is the empirically derived robust
c   estimate of the standard deviation of teleseismic travel-time
c   residuals (about 1.145 s).  Beta is the inverse square root of the
c   efficiency of the regression method times a correction for the
c   correlation matrix which is approximately the inverse square root
c   of the maximum weight (about 1.03).  This asymptotic value is
c   multiplied by an empirical non-asymptotic correction which turns
c   out to be nearly linear in the inverse square root of the number
c   of degrees of freedom.  As expected, the empirical factor is
c   independent of the number of parameters sought.  However, it is
c   unexpectedly slightly smaller than unity for large values of N.
c   For zero degrees of freedom, apparently due to non-linearity in
c   the free depth solution, the asymptotic correction depends on M.
c   Programmed on 17 March 1987 by R. Buland.
c
c   Why the empirical relationship should be slightly different for
c   held and free depth solutions is not exactly clear.  The
c   difference is not very large and may not be significant.
c   However, as it is consistent between 1 and 60 degrees of freedom
c   and as it is the best available information at this time, it has
c   been retained.
*/
double get_fact (int num, int nunk)
{
	double factor;

	if (num > nunk){             /* Positive degrees of freedom. */
		if (nunk > 3){
			factor = 1.94*( 0.926 + 0.333/sqrt(num-nunk) );
		}
		else{
			factor = 1.94*( 0.905 + 0.306/sqrt(num-nunk) );
		}
	}
	else{						/* Zero degrees of freedom. */
		if (nunk > 3){
			factor=5.72;
		}
		else{
			factor=2.64;
		}
	}
	return factor;
}

/* Subroutine to get error scaling factor.  Only used by calc_error.*/
/* Comments below come from reviser function fctrx2.f:				*/
/*		N is num, M is nunk											*/
/*  
c   Function fctrx2 returns a semi-empirical scaling factor which when
c   multiplied into the correlation matrix, yields the best current
c   estimate of the 90% confidence ellipsoid.  N is the number of
c   observations and M is the number of parameters being found (either
c   3 for a held depth or 4 for a free depth solution).  The factor is
c   asymptotically sigma times beta times the square root of chi
c   squared with M-1 degrees of freedom (for the spatial error
c   ellipsoid).  Sigma is the empirically derived robust estimate of the
c   standard deviation of teleseismic travel-time residuals (about
c   1.145 s).  Beta is the inverse square root of the efficiency of the
c   regression method times a correction for the correlation matrix
c   which is approximately the inverse square root of the maximum weight
c   (about 1.03).  This asymptotic value is multiplied by an empirical
c   non-asymptotic correction which turns out to be nearly linear in
c   the inverse square root of the number of degrees of freedom.  As
c   expected, the empirical factor is independent of the number of
c   parameters sought.  However, it is unexpectedly slightly smaller
c   than one for large values of N.  For zero degrees of freedom,
c   apparently due to non-linearity in the free depth solution, the
c   asymptotic correction depends on M.  Programmed on 17 March 1987
c   by R. Buland.

c   Why the empirical relationship should be slightly different for
c   held and free depth solutions is not exactly clear.  The
c   difference is not very large and may not be significant.
c   However, as it is consistent between 1 and 60 degrees of freedom
c   and as it is the best available information at this time, it has
c   been retained.
*/

double get_fact2 (int num, int nunk)
{
	double factor;

	if (num > nunk){             /* Positive degrees of freedom. */
		if (nunk > 3){
			factor = 2.94*( 0.926 + 0.333/sqrt(num-nunk) );
		}
		else{
			factor = 2.52*( 0.905 + 0.306/sqrt(num-nunk) );
		}
	}
	else{						/* Zero degrees of freedom. */
		if (nunk > 3){
			factor=8.67;
		}
		else{
			factor=3.43;
		}
	}
	return factor;
}
