#include "location.h"

/*#DOC  Title:																*/
/*#DOC    calc_gap															*/
/*#DOC  Desc:																*/
/*#DOC    Fills 'nsta, 'azimgap', and 'mindist' for hyps without them.		*/
/*#DOC  Input Arguments:													*/
/*#DOC    ep - pointer to structure containing event information.			*/
/*#DOC    h[] - array of structures containing hypocentres.					*/
/*#DOC    p[] - array of structures containing phases.						*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/
/*#DOC  Calls:																*/
/*#DOC  	calc_delta(), calc_esaz(), and dsort() from utils.c				*/

/*  Assign values to hypocentre variables used in ranking them as seeds.	*/
/*	If reporter provided nsta, azimgap, and mindist then use the values		*/
/*  given.  Otherwise need to calculate	the missing values.					*/

/*	Both delta and station-event azimuth are calculated for each station	*/
/*	regardless of whether this information is already in the database.		*/
/*	This is simpler and guarantees correctness.								*/

int calc_gap(struct event_rec *ep, struct hyp_rec *h, struct pha_rec *p)
{
	int i,j;
	int nsta;
	double *esaz;
	double delta,mindist;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* Assign memory for array of azimuths. */
	if ((esaz = (double*) calloc (ep->numphas, sizeof(double))) == NULL){
		sprintf(errstr,"calc_gap: evid %d: Memory problem.",ep->evid);
		return 1;
	}

	/* Loop over hypocentres in this event. */
	for (i=0; i<ep->numhyps; i++){

		if (h[i].nsta != NULLVAL && h[i].mindist != NULLVAL
		  && h[i].azimgap != NULLVAL)
			continue;

		if (diagnostic)
			fprintf(logfp,"calc_gap: %s ",h[i].agency);

		/* Loop over phases finding those reporter associated with this hyp. */
		nsta=0; mindist=180;
		for (j=0; j<ep->numphas; j++){
			if (p[j].hypid == h[i].hypid){

				/* For each reading increment nsta and			*/
				/* calculate values of azimuth and delta.	  	*/
				/* Store azimuths in local array for gap calc.	*/
				/* Check that delta isn't less than mindist.	*/
				if (p[j].init){
					delta = calc_delta(p[j].dircos,h[i].dircos);
					esaz[nsta++] = calc_esaz(p[j].dircos,h[i].dircos,
												p[j].sta_lon,h[i].lon,delta);

					/* Record lowest value of delta as mindist. */
					if (delta < mindist)
					 	mindist = delta;
				}
			}
		}
		if (nsta == 0)
			continue;

		if (h[i].nsta == NULLVAL)
			h[i].nsta = nsta;

		if (h[i].mindist == NULLVAL)
			h[i].mindist = mindist;

		if (diagnostic)
			fprintf(logfp,"nsta=%d min=%.1f\n",h[i].nsta,h[i].mindist);

		/* Use local array of azimuths filled above to calculate gap.	*/
		/* Only if reporter does not give a gap of their own.			*/
		if (h[i].nsta > 1 &&  h[i].azimgap == NULLVAL){

			dsort(esaz,0,nsta-1);
			h[i].azimgap = esaz[0] + 360 - esaz[nsta-1];

			for(j=0; j<nsta-1; j++)
				if( (esaz[j+1]-esaz[j]) > h[i].azimgap)
					h[i].azimgap = esaz[j+1]-esaz[j];

			if (diagnostic)
				fprintf(logfp,"calc_gap: gap=%.1f\n",h[i].azimgap);
		}
	}
	free(esaz);
	return 0;
}

