#include "location.h"

/*#DOC  Title:																*/
/*#DOC    calc_netmag														*/
/*#DOC  Desc:																*/
/*#DOC    Calculate MS and Mb for every station and average the results.	*/
/*#DOC  Input Arguments:													*/
/*#DOC    sp - pointer to structure containing current solution.			*/
/*#DOC    p[]  - array of phase structures.									*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/
/*#DOC  Calls:																*/
/*#DOC	 calc_sta_mb() from calc_sta_mb.c, calc_sta_ms() from calc_sta_ms.c	*/

int calc_netmag (struct sol_rec *sp, struct pha_rec p[])
{
	extern double mag_range_warn_thresh;			/* From config file */

	char prev_mb_sta[STALEN],prev_ms_sta[STALEN];
	double sta_mb,sta_ms;
	double reading_ms;
	double min_mb,min_ms,max_mb,max_ms;
	int n_pha_mb,n_read_ms;
	int prev_rdid;
	int reading[PHA_PER_READ];
	int i,j;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	n_pha_mb = n_read_ms = 0;	/* Number of mags for one station. */
	reading_ms = 0;				/* Average mag for one reading.	*/
	sta_mb   = sta_ms   = 0;	/* Average mag for one station. */
	sp->nsta_mb = sp->nsta_ms = 0;

	/* Set solution magnitudes to 0. */
	sp->bodymag = 0;
	sp->surfmag = 0;

	/* Keep track of max and min so can give range warning if needed. */
	min_mb = min_ms = 12;
	max_mb = max_ms = 0;

	/* MS calculation needs all the phases for a reading.  Store the		*/
	/* phase indexes in an array and call calc_sta_ms when got them all.	*/
	prev_rdid = 0;
	j = 0;					/* Number of phases for a reading. */

	/* Loop through all associated phases. */
	strcpy(prev_mb_sta,"");
	strcpy(prev_ms_sta,"");
	for (i=0; i<sp->numphas; i++){

		/* Set all station magnitudes to 0. */
		p[i].bodymag = 0;
		p[i].surfmag = 0;

		/* Do not consider purged phases. */
		if (p[i].purged)
			continue;

		/* Body wave magnitude. */
		calc_sta_mb(sp,&p[i]);
		if (p[i].bodymag){
			/* Check if same station as last one. */
			if (strcmp(p[i].sta,prev_mb_sta)){
				/* Finish off previous station. */
				if (sta_mb){
					sta_mb /= n_pha_mb;
					sp->bodymag += sta_mb;
					sp->nsta_mb++;
					if (sta_mb < min_mb) min_mb = sta_mb;
					if (sta_mb > max_mb) max_mb = sta_mb;
				}
				sta_mb = n_pha_mb = 0;
				strcpy(prev_mb_sta,p[i].sta);
			}
			sta_mb += p[i].bodymag;
			n_pha_mb++;
		}

		/* Surface wave magnitude - only done once per reading. */
		if (p[i].rdid != prev_rdid){
			if (j)
				reading_ms = calc_sta_ms(reading,j,p);
			if (reading_ms){
				if (strcmp(p[i].sta,prev_ms_sta)){
					/* Finish off previous station. */
					if (sta_ms){
						sta_ms /= n_read_ms;
						sp->surfmag += sta_ms;
						sp->nsta_ms++;
						if (sta_ms < min_ms) min_ms = sta_ms;
						if (sta_ms > max_ms) max_ms = sta_ms;
					}
					sta_ms = 0;
					n_read_ms = 0;
					strcpy(prev_ms_sta,p[i].sta);
				}
				sta_ms += reading_ms;
				n_read_ms++;
			}
			j=0;
			prev_rdid = p[i].rdid;
		}
		reading[j++] = i;

		/* Check not going past end of reading array. */
		if (j>PHA_PER_READ){
			sprintf(errstr,"calc_netmag: %s: too many phases",p[i].sta);
			return 1;
		}
	}
	/* Add last example of each and average over the contributing stations. */
	if (sta_mb){
		sta_mb /= n_pha_mb;
		sp->bodymag += sta_mb;
		sp->nsta_mb++;
		if (sta_mb < min_mb) min_mb = sta_mb;
		if (sta_mb > max_mb) max_mb = sta_mb;
		sp->bodymag /= sp->nsta_mb;

		if (diagnostic){
			fprintf(logfp,"calc_netmag: bodymag=%.2f ",sp->bodymag);
			fprintf(logfp,"nsta=%d\n",sp->nsta_mb);
		}

		/* Test for big differences in station magnitudes. */
		if ((max_mb - min_mb) > mag_range_warn_thresh)
			fprintf(logfp,"WARN: mb RANGE %.1f - %.1f\n",min_mb,max_mb);
	}
	if (sta_ms){
		sta_ms /= n_read_ms;
		sp->surfmag += sta_ms;
		sp->nsta_ms++;
		if (sta_ms < min_ms) min_ms = sta_ms;
		if (sta_ms > max_ms) max_ms = sta_ms;
		sp->surfmag /= sp->nsta_ms;

		if (diagnostic){
			fprintf(logfp,"calc_netmag: surfmag=%.2f ",sp->surfmag);
			fprintf(logfp,"nsta=%d\n",sp->nsta_ms);
		}

		/* Test for big differences in station magnitudes. */
		if ((max_ms - min_ms) > mag_range_warn_thresh)
			fprintf(logfp,"WARN: MS RANGE %.1f - %.1f\n",min_ms,max_ms);
	}
	return 0;
}
