#include "location.h"

/*#DOC  Title:																*/
/*#DOC    calc_pP_resid														*/
/*#DOC  Desc:																*/
/*#DOC    Fills 'pP_resid' for each pP phase.								*/
/*#DOC  Input Arguments:													*/
/*#DOC    sp - pointer to structure containing current solution.			*/
/*#DOC    p  - array of structures containing phases.						*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/
/*#DOC  Uses:																*/
/*#DOC	  read_pP_P() from read_pP_P.c										*/
/*#DOC	  add_to_error() and handle_error() from utils.c.					*/

int calc_pP_resid (struct sol_rec *sp, struct pha_rec p[])
{
	double p_time;
	char error_part[ERRLEN];
	int p_index  = 0;
	int i,j;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* Loop through all associated phases. */
	for (i=0; i<sp->numphas; i++){

		/* Set default value of resids to NULLVAL. */
		p[i].pP_resid = NULLVAL;

		/* Can't calculate residuals unless have a depth for the source. */
		if (sp->depth == NULLVAL)
			continue;

		/* Timeless phases are of no interest. */
		if (p[i].time == NULLVAL)
			continue;

		/* For each reading store P time in case there's a pP phase.	*/
		/* There can only be one P per reading.							*/
		if (strcmp(p[i].phase,"P") == 0){
			p_time = p[i].time;
			p_index   = i;
			continue;
		}

		/* If pP phase deal with it. */
		if (strcmp(p[i].phase,"pP") == 0){

			/* Check that the last P time stored is for the same reading. */
			if (p[i].rdid != p[p_index].rdid){
				sprintf(errstr,"calc_pP_resid:%s: pP without P",p[i].sta);
				handle_error();
				continue;
			}

			/* Don't want to use purged P phases. */
			if (p[p_index].purged)
				continue;

			/* Get pP-P time and corresponding dtdh from tables. */
			if ( read_pP_P(sp->depth, &p[i]) ){
				sprintf(error_part,"calc_pP_resid:%s:",p[i].sta);
				add_to_error(error_part);
				handle_error();
				continue;
			}

			/* Store residual between table pP - P and observed difference. */
			p[i].pP_resid = p[i].time - p_time - p[i].pP_P_time;

			if (diagnostic){
				fprintf(logfp,"calc_pP_resid: %5s %2s ",p[i].sta,p[i].phase);
				fprintf(logfp,"%0.3f %0.3f",sp->depth,p[i].delta);
				fprintf(logfp," pP-P=%f ",p[i].pP_P_time);
				fprintf(logfp,"dtdh=%f r=%f\n",p[i].pP_dtdh,p[i].pP_resid);
			}
		}
	}
	return 0;
}
