#include "location.h"

/*#DOC  Title:																*/
/*#DOC    calc_resid														*/
/*#DOC  Desc:																*/
/*#DOC    Fills 'ttime','resid','dtdh', and 'dtdd' fields for each phase.	*/
/*#DOC  Input Arguments:													*/
/*#DOC    sp - pointer to structure containing current solution.			*/
/*#DOC    p  - array of structures containing phases.						*/
/*#DOC    mode  - "all" if require resid for all phases.					*/
/*#DOC            "use" if only want resids that will be used in solution.	*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/
/*#DOC  Calls:																*/
/*#DOC	  read_ttime() from read_ttime.c									*/
/*#DOC	  add_to_error() and handle_error() from utils.c.					*/

int calc_resid (struct sol_rec *sp, struct pha_rec p[], char mode[4])
{
	extern double max_depth;		/* Set in file with ttime functions.	*/
	int all,status;
	double obtime;
	char error_part[ERRLEN];
	int i,j;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* Set flag to enable/disable residual calculation for unused phases. */
	if (strcmp(mode,"all")==0)
		all = 1;
	else if (strcmp(mode,"use")==0)
		all = 0;
	else{
		sprintf(errstr,"calc_resid: unrecognised mode %s.",mode);
		return 1;
	}

	/* Can't calculate residuals unless have a depth for the source. */
	if (sp->depth == NULLVAL){
		sprintf(errstr,"calc_resid: depthless hypocentre.");
		return 1;
	}

	/* Won't get residuals if source has gone too deep.		*/
	/* Want to fail without writing to stderr in this case. */
	if (sp->depth > max_depth){
		fprintf(logfp,"SOLUTION TOO DEEP\n");
		strcpy(errstr,"");
		return 1;
	}

	/* Loop through all associated phases. */
	for (i=0; i<sp->numphas; i++){

		/* Set default value of resids to NULLVAL. */
		p[i].resid = NULLVAL;

		/* To save time only calculate residuals for phases not used	*/
		/* in the solution (weight_factor=0) if requested to.			*/
		if (all==0 && p[i].weight_factor==0)
			continue;

		/* Timeless or codeless phases don't have residuals. */
		if (p[i].time == NULLVAL || !p[i].phase[0] )
			continue;

		/* pP,sP special case handled in different function.	*/
		if (strcmp(p[i].phase,"pP") == 0 || strcmp(p[i].phase,"sP") == 0)
			continue;

		/* Observed travel time for this phase. */
		obtime = p[i].time - sp->time;

		/* Update ttime, dtdd, and dtdh in phase structure. */
		/* Status 1 is a normal error.						*/
		/* Status 2 is a warning message.					*/
		/* Status 3 means want to set phase ID to null.		*/
		status = read_ttime(sp, &p[i]);
		if (status){
			sprintf(error_part,"calc_resid: %s %s:",p[i].sta,p[i].phase);
			add_to_error(error_part);
			if (status == 1)
				handle_error();
			else if (status == 2){
				fprintf(logfp,"WARN: %s\n",errstr);
			}
			else if (status == 3){
				fprintf(logfp,"WARN: %s\n",errstr);
				strcpy(p[i].phase,"");
			}
			p[i].weight_factor = 0;
			continue;
		}

		/* Calculate residual for this phase. */
		p[i].resid = obtime - p[i].ttime;

		if (diagnostic){
			fprintf(logfp,"calc_resid: %5s %2s ",p[i].sta,p[i].phase);
			fprintf(logfp,"%0.3f tt=%f ",p[i].delta,p[i].ttime);
			fprintf(logfp,"dtdd=%f dtdh=%f ",p[i].dtdd,p[i].dtdh);
			fprintf(logfp,"r=%f\n",p[i].resid);
		}
	}
	return 0;
}
