#include "location.h"

/*#DOC  Title:																*/
/*#DOC    calc_sta_ms														*/
/*#DOC  Desc:																*/
/*#DOC    Calculate MS for a single reading and update the phases used.		*/
/*#DOC  Input Arguments:													*/
/*#DOC    reading - array of p[] indexes for one reading.					*/
/*#DOC    n   - size of reading array.										*/
/*#DOC    p[] - array of phase structures.									*/
/*#DOC  Return:																*/
/*#DOC    MS for this reading. Returned as won't be on every phase.			*/

/* For maximum flexibility treat whole reading at once so can combine		*/
/* amplitudes from different components, whether these are  attached to		*/
/* seperate, amplitude only, phases or all held in amp array of one phase.	*/

/* Calculate MSZ and MSH seperately and then average them to get station	*/
/* magnitude. Keep the corresponding magnitude in phase structure(s) of		*/
/* the phase(s) that were used to calculate it.								*/

/* Reviser calculates stamags for delta>5 but only uses them for netmag		*/
/* when delta>20.  Use all stamags for netmag here so for same magnitude	*/
/* as reviser need to set surf_mag_min_dist to 20.							*/

double calc_sta_ms (int reading[], int n, struct pha_rec p[])
{
	extern double surf_mag_min_dist;		/* From config file */
	extern double surf_mag_max_dist;		/* From config file */
	extern double surf_mag_min_per;			/* From config file */
	extern double surf_mag_max_per;			/* From config file */

	double reading_ms;
	double minper,maxper;
	double z_amp,n_amp,e_amp;
	double z_per,n_per,e_per;
	double amp,per;
	double msh,msz;
	int z_phase,n_phase,e_phase;
	int i,j;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	reading_ms = 0;
	minper = surf_mag_min_per;
	maxper = surf_mag_max_per;
	amp = z_amp = n_amp = e_amp = 0;
	per = z_per = n_per = e_per = 0;

	/* Loop over phases in this reading. */
	for (i=reading[0]; i <= reading[n-1]; i++){

		/* Need to consider phases without ISC phase code if amplitude	*/
		/* measured at right period.									*/
		if (strcmp(p[i].phase,"") == 0 && p[i].numamps > 0 && 
			p[i].delta>=surf_mag_min_dist && p[i].delta<=surf_mag_max_dist ){

			/* Loop over amplitudes for this phase. */
			/* Adopt first period and reject others if 5 sec different.	*/
			for (j=0; j<p[i].numamps; j++){
				if (p[i].a[j].per >= minper && p[i].a[j].per <= maxper ){

					if (p[i].a[j].amp == 0 || p[i].a[j].amp == NULLVAL)
						continue;

					/* Only amps from known component are any use. */
					if (!p[i].a[j].comp)
						continue;

					/* Take first example of each component. */
					if (p[i].a[j].comp == 'Z' && !z_amp){
						z_amp = p[i].a[j].amp;
						z_per = p[i].a[j].per;
						z_phase = i;
					}
					else if (p[i].a[j].comp == 'N' && !n_amp){
						n_amp = p[i].a[j].amp;
						n_per = p[i].a[j].per;
						n_phase = i;
					}
					else if (p[i].a[j].comp == 'E' && !e_amp){
						e_amp = p[i].a[j].amp;
						e_per = p[i].a[j].per;
						e_phase = i;
					}
					minper = p[i].a[j].per - 5;
					if (minper < surf_mag_min_per) minper = surf_mag_min_per;
					maxper = p[i].a[j].per + 5;
					if (maxper > surf_mag_max_per) maxper = surf_mag_max_per;
				}
			}
		}
	}
	/* Need either vertical or full horizontal or both. */
	if (n_amp && e_amp){
		amp = sqrt(n_amp*n_amp + e_amp*e_amp);
		per = (n_per + e_per)/2;
		msh = log10(amp/per) + 1.66* log10(p[reading[0]].delta) + 0.3;

		if (diagnostic){
			fprintf(logfp,"calc_sta_ms: %s N and E",p[reading[0]].sta);
			fprintf(logfp,"amp=%.2f per=%.2f msh=%.2f\n",amp,per,msh);
		}

		p[n_phase].surfmag = msh;
		p[e_phase].surfmag = msh;

		reading_ms = msh;
	}
	if (z_amp){

		msz = log10(z_amp/z_per) + 1.66* log10(p[reading[0]].delta) + 0.3;

		if (diagnostic){
			fprintf(logfp,"calc_sta_ms: %s Z",p[reading[0]].sta);
			fprintf(logfp,"amp=%.2f per=%.2f msz=%.2f\n",amp,per,msz);
		}
		p[z_phase].surfmag = msz;

		if (reading_ms)
			reading_ms = (msh + msz)/2;
			if (z_phase == n_phase)
				p[z_phase].surfmag = reading_ms;
		else
			reading_ms = msz;
	}
	return reading_ms;
}
