#include "location.h"

/*#DOC  Title:																*/
/*#DOC    huber_weight														*/
/*#DOC  Desc:																*/
/*#DOC    Fills 'alpha' in solution structure and 'weight' for each phase.	*/
/*#DOC  Input Arguments:													*/
/*#DOC    sp - pointer to structure containing current solution.			*/
/*#DOC    p  - array of structures containing phases.						*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/

/* Simple weighting designed to get basic solution before using more	*/
/* complex but less stable 'Buland' weighting to get final solution.	*/

/* From Fortran code (nwtfcn used to switch between Huber and Jeffreys):

c   When nwtfcn < 0, function wght returns the Huber-t weight for
c   residual r in seconds.  The cutoff, s, between least squares
c   behavior and one-norm behavior has been determined empirically to
c   optimize efficiency against the distribution of teleseismic
c   travel-time residuals.  For this case, the efficiency is about
c   .905 which is quite close to the optimal efficiency achievable
c   with a non-decreasing influence function.  Except for the
c   non-linearity inherent in the earthquake location problem, this
c   weighting would always produce unique results.  When nwtfcn >= 0,
c   function wght returns the Jeffreys weight using mu of .05 and
c   sigma of 1.145.  This should result in efficiency in excess of .99.
c   However, the result is not unique unless started from the converged
c   result of the first weighting.  Programmed on 17 March 1987 by
c   R. Buland.

*/

#define CUT_OFF 0.8275	/* From Buland's Fortran */

int huber_weight (struct sol_rec *sp, struct pha_rec p[])
{
	extern int    min_phases;			/* From config file */
	extern double init_max_resid;		/* From config file */
	extern double max_resid;			/* From config file */

	int i,ndef;
	double spread;
	double total_weight;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* First iteration:												*/
	/* Initialise alpha as average residual.						*/
	/* Then calculate initial weights.								*/
	/* Afterwards, calculate weights again as usual.				*/
	if (sp->iteration == 0){
		sp->alpha=0;
		ndef=0;

		for (i=0; i<sp->numphas; i++){
			if (p[i].weight_factor > 0 && fabs(p[i].resid) < init_max_resid){
				sp->alpha += p[i].resid;
				ndef++;

			}
			if (diagnostic){
				fprintf(logfp,"huber_weight: init: %s %s ",p[i].sta,p[i].phase);
				fprintf(logfp,"%f %f\n",p[i].weight_factor,p[i].resid);
			}
		}
		if (ndef < min_phases ){
			fprintf(logfp,"TOO FEW PHASES huber_weight init: ndef=%d\n",ndef);
			strcpy(errstr,"");
			return 1;
		}
		sp->alpha /= ndef;
		sp->prev_alpha = sp->alpha;

		if (diagnostic)
			fprintf(logfp,"huber_weight: init alpha=%f n=%d ",sp->alpha,ndef);

		for (i=0; i<sp->numphas; i++){
			if (fabs(p[i].resid) < max_resid){
				spread = fabs(p[i].resid - sp->alpha);
				if (spread <= CUT_OFF)
					p[i].weight = 1;
				else
					p[i].weight = CUT_OFF/spread;

				p[i].weight *= p[i].weight_factor;
			}
			else
				p[i].weight = 0;
		}
	}

	/* First calculate alpha. */
	sp->alpha=0;
	total_weight=0;
	ndef=0;
	for (i=0; i<sp->numphas; i++){
		if (p[i].weight_factor > 0 && fabs(p[i].resid) < max_resid){
			sp->alpha += p[i].resid*p[i].weight;
			total_weight += p[i].weight;
			ndef++;
		}
		if (diagnostic){
			fprintf(logfp,"huber_weight: %s %s ",p[i].sta,p[i].phase);
			fprintf(logfp,"%f %f ",p[i].weight_factor,p[i].weight);
			fprintf(logfp,"%f\n",p[i].resid);
		}
	}
	if (ndef < min_phases ){
		fprintf(logfp,"TOO FEW PHASES huber_weight: ndef=%d\n",ndef);
		strcpy(errstr,"");
		return 1;
	}
	sp->alpha /= total_weight;

	if (diagnostic){
		fprintf(logfp,"huber_weight: alpha=%f ndef=%d ",sp->alpha,ndef);
		fprintf(logfp,"total_weight=%f\n",total_weight);
	}

	/* Then calculate weights. */
	for (i=0; i<sp->numphas; i++){
		if (fabs(p[i].resid) < max_resid){
			spread = fabs(p[i].resid - sp->alpha);
			if (spread <= CUT_OFF)
				p[i].weight = 1;
			else
				p[i].weight = CUT_OFF/spread;

			p[i].weight *= p[i].weight_factor;
		}
		else
			p[i].weight = 0;
	}
	return 0;
}
