#include "location.h"
#include "iaspei_model.h"

/*#DOC  Title:																*/
/*#DOC    id_iaspei															*/
/*#DOC  Desc:																*/
/*#DOC    Alters the 'phase' field of a phase if it does not fit IASP91.	*/
/*#DOC  Input Arguments:													*/
/*#DOC    sp - pointer to structure containing current solution.			*/
/*#DOC    pp - pointer to a phase structure.								*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/
/*#DOC  Uses:																*/
/*#DOC    read_ttime() from read_ttime.c									*/

/* Forget about the various PKP branches for now. IASPEI handles them in	*/
/* more detail than JB but unless they're going to be given weights			*/
/* it doesn't seem worthwhile messing with them.							*/

int id_iaspei(struct sol_rec *sp, struct pha_rec *pp)
{
	extern double max_resid;			/* From config file */

	double g_time,b_time,n_time;
	double p_time,pn_time,s_time,sn_time;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* If source is too deep any call to read_ttime will give error. */
	if (sp->depth > MAX_DEPTH)
		return 0;

	/* Check for Pdiff that's realy P. */
	if (strcmp(pp->phase,"Pdiff") ==0)
		if (pp->delta < PDIFF_MIN_DELTA)
			strcpy(pp->phase,"P");
	
	/* Get Pn and P at some depth/delta combinations.  Assume that	*/
	/* arrival labeled P is the first arriving of P and Pn.			*/
	if (strcmp(pp->phase,"P") ==0 || strcmp(pp->phase,"Pg")==0 ||
		strcmp(pp->phase,"Pb")==0 || strcmp(pp->phase,"Pn")==0 ){

		/* Devide into crustal and non-crustal. Pn could be either. */
		if (pp->delta > CRUSTAL_P_DELTA || sp->depth > MOHO){

			/* Check whether P or Pn if close enough. */
			if (pp->delta < PN_MAX_DELTA){
				strcpy(pp->phase,"P");
				if( read_ttime(sp, pp) ){
					strcpy(errstr,"");
					pp->ttime = -NULLVAL;
				}
				p_time = pp->ttime;

				strcpy(pp->phase,"Pn");
				if( read_ttime(sp, pp) ){
					strcpy(errstr,"");
					pp->ttime = -NULLVAL;
				}
				pn_time = pp->ttime;

				if (p_time == pn_time == -NULLVAL)
					strcpy(pp->phase,"");
				else if (p_time < pn_time)
					strcpy(pp->phase,"P");
			}
			/* Check for Pdiff. */
			else if (pp->delta >= PDIFF_MIN_DELTA){
				strcpy(pp->phase,"Pdiff");
				if (read_ttime(sp, pp) )
					strcpy(pp->phase,"P");
				else if (fabs(pp->time-sp->time-pp->ttime) > max_resid)
					strcpy(pp->phase,"");
			}
			else
				strcpy(pp->phase,"P");

			if (strcmp(pp->phase,"P")==0 && pp->delta > P_MAX_DELTA)
				strcpy(pp->phase,"");

		}
		/* Crustal phase. */
		else{
			/* In upper crust - could be Pg, Pb, or Pn. */
			if (sp->depth < CONRAD){
				/* If reported as P then assume its first arriving phase. */ 
				if (strcmp(pp->phase,"P")==0){

					/* Pg */
					strcpy(pp->phase,"Pg");
					if( read_ttime(sp, pp) ){
						strcpy(errstr,"");
						pp->ttime = -NULLVAL;
					}
					g_time = pp->ttime;

					/* Pb */
					strcpy(pp->phase,"Pb");
					if( read_ttime(sp, pp) ){
						strcpy(errstr,"");
						pp->ttime = -NULLVAL;
					}
					b_time = pp->ttime;

					/* Pn */
					strcpy(pp->phase,"Pn");
					if( read_ttime(sp, pp) ){
						strcpy(errstr,"");
						pp->ttime = -NULLVAL;
					}
					n_time = pp->ttime;

					if (g_time == b_time == n_time == -NULLVAL)
						strcpy(pp->phase,"");
					else if (g_time <= b_time && g_time < n_time)
						strcpy(pp->phase,"Pg");
					else if (b_time < g_time && b_time < n_time)
						strcpy(pp->phase,"Pb");
					else
						strcpy(pp->phase,"Pn");

					if (diagnostic){
						fprintf(logfp,"id_iaspei: del=%f ",pp->delta);
						fprintf(logfp,"z=%f g=%f ",sp->depth,g_time);
						fprintf(logfp,"b=%f n=%f\n",b_time,n_time);
					}
				}
				/* Otherwise check that reported phase is possible. */
				else
					if( read_ttime(sp, pp) ){
						strcpy(errstr,"");
						strcpy(pp->phase,"");
					}
			}
			/* In lower crust - could be Pb or Pn. */
			else{
				/* If reported P or Pg assume it's first arriving phase. */ 
				if(strcmp(pp->phase,"P")==0 || strcmp(pp->phase,"Pg")==0){

					/* Pb */
					strcpy(pp->phase,"Pb");
					if( read_ttime(sp, pp) ){
						strcpy(errstr,"");
						pp->ttime = -NULLVAL;
					}
					b_time = pp->ttime;

					/* Pn */
					strcpy(pp->phase,"Pn");
					if( read_ttime(sp, pp) ){
						strcpy(errstr,"");
						pp->ttime = -NULLVAL;
					}
					n_time = pp->ttime;

					if (b_time == n_time == -NULLVAL)
						strcpy(pp->phase,"");
					else if (b_time < n_time)
						strcpy(pp->phase,"Pb");
					else
						strcpy(pp->phase,"Pn");

					if (diagnostic){
						fprintf(logfp,"id_iaspei: del=%f ",pp->delta);
						fprintf(logfp,"z=%f ",sp->depth);
						fprintf(logfp,"b=%f n=%f\n",b_time,n_time);
					}
				}
				/* Otherwise check that reported phase is possible. */
				else
					if( read_ttime(sp, pp) ){
						strcpy(errstr,"");
						strcpy(pp->phase,"");
					}
			}
		}		/* above moho, delta < CRUSTAL_P_DELTA */
	}		/* P phase */

	/* Check for Sdiff that's realy S. */
	if (strcmp(pp->phase,"Sdiff") ==0)
		if (pp->delta < SDIFF_MIN_DELTA)
			strcpy(pp->phase,"S");

	if (strcmp(pp->phase,"S") ==0 || strcmp(pp->phase,"Sg")==0 ||
		strcmp(pp->phase,"Sb")==0 || strcmp(pp->phase,"Sn")==0 ){

		/* Devide into crustal and non-crustal */
		if (pp->delta > CRUSTAL_S_DELTA || sp->depth > MOHO){

			/* Check whether S or Sn if close enough. */
			if (pp->delta < SN_MAX_DELTA){
				strcpy(pp->phase,"S");
				if( read_ttime(sp, pp) ){
					strcpy(errstr,"");
					pp->ttime = -NULLVAL;
				}
				s_time = pp->ttime;

				strcpy(pp->phase,"Sn");
				if( read_ttime(sp, pp) ){
					strcpy(errstr,"");
					pp->ttime = -NULLVAL;
				}
				sn_time = pp->ttime;

				if (s_time == sn_time == -NULLVAL)
					strcpy(pp->phase,"");
				else if (s_time < sn_time)
					strcpy(pp->phase,"S");
			}
			/* Check for Sdiff. */
			else if (pp->delta >= SDIFF_MIN_DELTA){
				strcpy(pp->phase,"Sdiff");
				if (read_ttime(sp, pp) )
					strcpy(pp->phase,"S");
				else if (fabs(pp->time-sp->time-pp->ttime) > max_resid)
					strcpy(pp->phase,"");
			}
			else
				strcpy(pp->phase,"S");

			if (strcmp(pp->phase,"S")==0 && pp->delta > S_MAX_DELTA)
				strcpy(pp->phase,"");
		}
		/* Crustal phase. */
		else{
			/* In upper crust - could be Sg, Sb, Sn. */
			if (sp->depth < CONRAD){
				/* If reported as S then assume it's first arriving phase. */ 
				if (strcmp(pp->phase,"S")==0){

					/* Sg */
					strcpy(pp->phase,"Sg");
					if( read_ttime(sp, pp) ){
						strcpy(errstr,"");
						pp->ttime = -NULLVAL;
					}
					g_time = pp->ttime;

					/* Sb */
					strcpy(pp->phase,"Sb");
					if( read_ttime(sp, pp) ){
						strcpy(errstr,"");
						pp->ttime = -NULLVAL;
					}
					b_time = pp->ttime;

					/* Sn */
					strcpy(pp->phase,"Sn");
					if( read_ttime(sp, pp) ){
						strcpy(errstr,"");
						pp->ttime = -NULLVAL;
					}
					n_time = pp->ttime;

					if (g_time == b_time == n_time == -NULLVAL)
						strcpy(pp->phase,"");
					else if (g_time <= b_time && g_time < n_time)
						strcpy(pp->phase,"Sg");
					else if (b_time < g_time && b_time < n_time)
						strcpy(pp->phase,"Sb");
					else
						strcpy(pp->phase,"Sn");

					if (diagnostic){
						fprintf(logfp,"id_iaspei: del=%f ",pp->delta);
						fprintf(logfp,"z=%f g=%f ",sp->depth,g_time);
						fprintf(logfp,"b=%f n=%f\n",b_time,n_time);
					}
				}
				/* Otherwise check that reported phase is possible. */
				else
					if( read_ttime(sp, pp) ){
						strcpy(errstr,"");
						strcpy(pp->phase,"");
					}
			}
			/* In lower crust - could be Sb or Sn. */
			else{
				/* If reported S or Sg assume it's first arriving phase. */ 
				if(strcmp(pp->phase,"S")==0 || strcmp(pp->phase,"Sg")==0){

					/* Sb */
					strcpy(pp->phase,"Sb");
					if( read_ttime(sp, pp) ){
						strcpy(errstr,"");
						pp->ttime = -NULLVAL;
					}
					b_time = pp->ttime;

					/* Sn */
					strcpy(pp->phase,"Sn");
					if( read_ttime(sp, pp) ){
						strcpy(errstr,"");
						pp->ttime = -NULLVAL;
					}
					n_time = pp->ttime;

					if (b_time == n_time == -NULLVAL)
						strcpy(pp->phase,"");
					else if (b_time < n_time)
						strcpy(pp->phase,"Sb");
					else
						strcpy(pp->phase,"Sn");

					if (diagnostic){
						fprintf(logfp,"id_iaspei: del=%f ",pp->delta);
						fprintf(logfp,"z=%f ",sp->depth);
						fprintf(logfp,"b=%f n=%f\n",b_time,n_time);
					}
				}
				/* Otherwise check that reported phase is possible. */
				else
					if( read_ttime(sp, pp) ){
						strcpy(errstr,"");
						strcpy(pp->phase,"");
					}
			}

		}		/* above moho delta < CRUSTAL_S_DELTA */
	}		/* s_phase true */

	return 0;
}
