#include "location.h"

#define NUM_PHASE_MAP 40				 /* Must match structure below. */

struct phase_map_rec {
	char rep_phase[PHALEN];
	char phase[PHALEN];
};

struct phase_map_rec phase_map[NUM_PHASE_MAP] = {
	{"p" , "P" },
	{"P" , "P" },
	{"pP", "pP"},
	{"AP", "pP"},
	{"Pg", "Pg"},
	{"P*", "Pb"},
	{"Pb", "Pb"},
	{"Pn", "Pn"},
	{"PG", "Pg"},
	{"PB", "Pb"},
	{"PN", "Pn"},
	{"PKP","PKP"},
	{"P/PKP","P"},
	{"Pdiff","Pdiff"},
	{"PDIF","Pdiff"},
	{"s" , "S" },
	{"S" , "S" },
	{"Sg", "Sg"},
	{"S*", "Sb"},
	{"Sb", "Sb"},
	{"Sn", "Sn"},
	{"SG", "Sg"},
	{"SB", "Sb"},
	{"SN", "Sn"},
	{"SKS","SKS"},
	{"S/(SKS)","S"},
	{"PcP","PcP"},
	{"ScS","ScS"},
	{"PP" ,"PP"},
	{"PPP","PPP"},
	{"PS" ,"PS"},
	{"SP" ,"SP"},
	{"PPS","PPS"},
	{"SPP","SPP"},
	{"PSP","PSP"},
	{"SS" ,"SS"},
	{"SSP","SSP"},
	{"PSS","PSS"},
	{"SPS","SPS"},
	{"SSS","SSS"},
};

/*#DOC  Title:																*/
/*#DOC    id_pha															*/
/*#DOC  Desc:																*/
/*#DOC    Fills the 'phase' field of each phase structure with an ISC code.	*/
/*#DOC  Input Arguments:													*/
/*#DOC    sp  - pointer to structure containing current solution.			*/
/*#DOC    p[] - array of phase structures.									*/
/*#DOC    reid_phase -	flag from event structure.							*/
/*#DOC    				Will be 1 unless set to 0 on instruction line.		*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/
/*#DOC  Uses:																*/
/*#DOC    id_jb() from id_jb.c												*/
/*#DOC    handle_error() from utils.c												*/

/* Uses 'rep_phase' field in phase structure to set the 'phase' field.		*/
/* Ensures that the codes used conform to ISC standard in case etc.			*/
/* The phase_map structure is used to recognise and possibly change codes.	*/
/* Sets phase to null if rep_phase is not recognised.						*/
/* Assumes unidentified first phases are 'P'.								*/
/* Gives non-initial 'P' phases a null isc_code.							*/
/* Reidentifies phases if given code is impossible, given the current		*/
/* solution's depth and delta values.										*/

/* This function is run each iteration. It needs to be run after:			*/
/*		calc_delaz															*/

int id_pha (struct sol_rec *sp, struct pha_rec p[], int reid_phase)
{
	int i,j;

	/* This function would need changing if the model were changed. */
	int id_jb (struct sol_rec *sp, struct pha_rec p[]);

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	for (i=0; i<sp->numphas; i++){

		if (diagnostic)
			fprintf(logfp,"id_pha:%s rep_phase=%s\n",p[i].sta,p[i].rep_phase);

		/* Set ISC phase equal to null. */
		strcpy(p[i].phase,"");

		/* Don't give phases without a time an ISC code. */
		if (p[i].time == NULLVAL)
			continue;

		/* Assume unnamed initial phases are P. */
		if (strcmp(p[i].rep_phase,"")==0 && p[i].init)
			strcpy(p[i].phase,"P");

		/* Map alternate spellings/codes to ISC standard */
		for (j=0;j<NUM_PHASE_MAP;j++){
			if (strcmp(p[i].rep_phase,phase_map[j].rep_phase)==0){
				strcpy(p[i].phase,phase_map[j].phase);
				break;
			}
		}
		if (diagnostic)
			fprintf (logfp,"id_pha:%s mapped phase=%s\n",p[i].sta,p[i].phase);

		/* Check for non-initial P phases. */
		/* If an agency gives two Ps in the same reading it causes problems. */
		if (strcmp(p[i].phase,"P")==0 && !p[i].init)
			strcpy(p[i].phase,"");

		/* Unless option was chosen not to reconsider phase codes check */
		/* that reporters phase is possible for current depth/delta and	*/
		/* rename if necessary. Function used is model dependent. 		*/
		if (reid_phase)
			if (id_jb(sp,&p[i]))
				handle_error();
			
		if (diagnostic)
			fprintf (logfp,"id_pha:%s phase=%s\n",p[i].sta,p[i].phase);

	}		/* phase loop */
	return 0;
}
