#include "location.h"

/*#DOC  Title:																*/
/*#DOC    init_event														*/
/*#DOC  Desc:																*/
/*#DOC    Deals with depth_agency/location_agency/time_agency instructions.	*/
/*#DOC    Sets event type depending on reported event types.				*/
/*#DOC  Input Arguments:													*/
/*#DOC    ep - pointer to structure containing event information.			*/
/*#DOC    h[] - array of hypocentre structures.								*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/


/* If a depth/location agency was given then assume that want to start with */
/* this agency's hypocentre as seed, unless a seed_agency was also given.	*/

int init_event (struct event_rec *ep, struct hyp_rec h[])
{
	extern double default_depth;	/* From config file */
	extern int min_phases;			/* From config file */
	int severity,max_severity;
	int i;

	/* Check that there are enough phases to start with.	*/
	/* If not then skip event without writing to stderr.	*/
	if (ep->numphas < min_phases){
		fprintf(logfp,"init_event: %d: only %d phases",ep->evid,ep->numphas);
		strcpy(errstr,"");
		return 1;
	}

	/* depth_agency instruction  - fix depth to that given by chosen agency. */
	if (ep->depth_agency[0]){
		for (i=0; i< ep->numhyps; i++){
			if (strcmp(h[i].agency,ep->depth_agency)==0){
				ep->fixed_depth = h[i].depth;
				break;
			}
		}
		if (ep->fixed_depth == NULLVAL){
			sprintf(errstr,"init_event: %d: no depth_agency hyp",ep->evid);
			return 1;
		}
		if (ep->seed_agency[0] == '\0')
			strcpy(ep->seed_agency,ep->depth_agency);
	}

	/* location_agency instruction  - fix lat,lon to that given by agency.	*/
	/* Unless fixed seperately also fix depth to that given by agency.		*/
	if (ep->location_agency[0]){

		for (i=0; i< ep->numhyps; i++){
			if (strcmp(h[i].agency,ep->location_agency)==0){
				ep->fixed_lat = h[i].lat;
				ep->fixed_lon = h[i].lon;
				break;
			}
		}
		if (ep->fixed_lat == NULLVAL){
			sprintf(errstr,"init_event: %d: no location_agency hyp",ep->evid);
			return 1;
		}
		if (ep->fixed_depth == NULLVAL)
			ep->fixed_depth = h[i].depth;

		if (ep->seed_agency[0] == '\0')
			strcpy(ep->seed_agency,ep->location_agency);
	}

	/* time_agency instruction  - fix time to that given by agency.			*/
	/* Unless fixed seperately also fix lat,lon to that given by agency.	*/
	/* Unless fixed seperately also fix depth to that given by agency.		*/
	if (ep->time_agency[0]){
		for (i=0; i< ep->numhyps; i++){
			if (strcmp(h[i].agency,ep->time_agency)==0){
				ep->fixed_time = h[i].time;
				break;
			}
		}
		if (ep->fixed_time == NULLVAL){
			sprintf(errstr,"init_event: %d: no time_agency hyp",ep->evid);
			return 1;
		}
		if (ep->fixed_lat == NULLVAL){
			ep->fixed_lat = h[i].lat;
			ep->fixed_lon = h[i].lon;
		}
		if (ep->fixed_depth == NULLVAL)
			ep->fixed_depth = h[i].depth;

		if (ep->seed_agency[0] == '\0')
			strcpy(ep->seed_agency,ep->location_agency);
	}
	/* If lat/lon fixed but not depth then fix depth to default value.	*/
	/* Can happen either if lat, lon fixed explicitly but not depth or	*/
	/* if agency used to fix location does not supply a depth.			*/
	if (ep->fixed_lat != NULLVAL && ep->fixed_depth == NULLVAL)
			ep->fixed_depth = default_depth;

	/* If time fixed explicitly but not lat/lon then an error. */
	if (ep->fixed_time != NULLVAL && ep->fixed_lat == NULLVAL){
		sprintf(errstr,"init_event: %d: time fixed but not location",ep->evid);
		return 1;
	}

	/* Go through etypes for reported hypocentres,	*/
	/*  setting ISC etype to most severe. 			*/
	ep->etype[0]='\0';
	max_severity = 0;
	for (i=0; i < ep->numhyps; i++){
		severity = 0;

		if (!h[i].etype)
			continue;

		/* Known / suspected character. */
		if ( h[i].etype[0] == 'd')
			severity = 40;
		else if ( h[i].etype[0] == 'f')
			severity = 30;
		else if ( h[i].etype[0] == 'k')
			severity = 20;
		else if ( h[i].etype[0] == 's')
			severity = 10;

		/* Event type character. */
		if ( h[i].etype[1] == 'n')
			severity += 9;
		else if ( h[i].etype[1] == 'r')
			severity += 8;
		else if ( h[i].etype[1] == 'm')
			severity += 7;
		else if ( h[i].etype[1] == 'x')
			severity += 6;
		else if ( h[i].etype[1] == 'h')
			severity += 5;

		if (severity > max_severity){
			max_severity=severity;
			strcpy(ep->etype,h[i].etype);
		}
	}
	return 0;
}
