#include "location.h"

/*#DOC  Title:																*/
/*#DOC    mark_duplicates													*/
/*#DOC  Desc:																*/
/*#DOC    Seperates phase array into readings and passes these on.			*/
/*#DOC  Input Arguments:													*/
/*#DOC    sp - pointer to structure containing current solution.			*/
/*#DOC    p  - array of pointers to structures containing phases.			*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/
/*#DOC  Calls:																*/
/*#DOC	  same_sta() from this file.										*/
/*#DOC    print_pha() from utils.c											*/

/* Goes through array of phase structures (p) and creates an array	*/
/* of indexes (samesta) for each station. Passes these arrays to 	*/
/* function same_sta to be checked for duplicates.					*/
/* Passed pointer to solution structure sp so that can use numphas.	*/
/* Passes on this pointer so that same_time can calculate resids. 	*/
/* Uses fact that phases are sorted by sta already.		*/

/* Needs id_pha and get_weight_factor to have been run already.		*/

/* Local functions called only within this file. */
int same_sta (int samesta[], int n, struct sol_rec* sp, struct pha_rec p[]);
int same_pha (int samepha[], int n, struct pha_rec p[]);
int same_time (int sametime[], int n, struct sol_rec* sp, struct pha_rec p[]);

int mark_duplicates(struct sol_rec* sp, struct pha_rec p[])
{
	int samesta[PHA_PER_READ];
	int i,j,k;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* Create an array samesta of phase indexes for each station */
	/* and pass it on to the next stage. */
	for (i=0; i<sp->numphas; i+=j){
		j=0;		/* Number of phases with this sta. */

		/* Not interested in phases that have already been discounted.	*/
		/* May have null phase codes or times.							*/
		/* Should consider purged phases tho' or their duplicates will	*/
		/* be re-instated.												*/
		if (p[i].weight_factor == 0 && !p[i].purged){
			i++;
			continue;
		}

		for (k=i; k<sp->numphas; k++){

			if (strcmp(p[k].sta,p[i].sta) || strcmp(p[k].net,p[i].net))
				break;

			/* Reset duplicate flags every iteration. */
			p[k].duplicate = 0;

			/* Not interested in phases that have already been discounted.	*/
			/* May have null phase codes or times.							*/
			if (p[k].weight_factor == 0 && !p[k].purged){
				i++;
				continue;
			}

			samesta[j++] = k;

			/* Check not going past end of samesta array. */
			if (j>PHA_PER_READ){
				sprintf(errstr,"mark_duplicates: %s: too many phases",
				        p[i].sta);
				return 1;
			}
		}
		/* Look for duplicates. */
		if ( same_sta(samesta,j,sp,p) )
			return 1;
	}

	if (diagnostic){
		fprintf(logfp,"mark_duplicates: done\n");
		print_pha(sp->numphas, p);
	}
	return 0;
}

/*#DOC  Title:																*/
/*#DOC    same_sta    ( called only from mark_duplicates() )				*/
/*#DOC  Desc:																*/
/*#DOC    Creates arrays for same_time() and same_pha()	before calling them.*/
/*#DOC  Input Arguments:													*/
/*#DOC    samesta - array of p[] indexes for one reading.					*/
/*#DOC    n   - size of samesta array.										*/
/*#DOC    sp  - pointer to structure containing current solution.			*/
/*#DOC    p[] - array of phase structures.									*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/
/*#DOC  Calls:																*/
/*#DOC	  same_time() and same_pha() from this file.						*/

/* Receives an array of phase structure indexes for a single		*/
/* station (samesta) and the size of this array (n).				*/
/* First creates an index array for each time present (sametime)	*/
/* and passes these arrays to function diff_pha to make sure that	*/
/* only one phase code is given for each time.						*/
/* Then creates an index array for each phase code (samepha)		*/
/* and sends each one to function same_pha to be dealt with.		*/
/* Pointer to solution structure sp is not used here, just passed	*/
/* on to same_time, which needs it to calculate residuals.			*/

int same_sta (int samesta[], int n, struct sol_rec* sp, struct pha_rec p[])
{
	int sametime[PHA_PER_READ];
	int samepha[PHA_PER_READ];
	int done[PHA_PER_READ];
	int i,j,k;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	if (diagnostic){
		fprintf(logfp,"same_sta: %s ",p[samesta[0]].sta);
		fprintf(logfp,"%d phases\n",n);
	}

	/* First check for phases with different codes but the same time. */

	/* Array of flags parallel to samesta array to show that	*/
	/* that phase's time has already been dealt with.			*/
	for (i=0; i<n; i++)
		done[i]=0;

	/* Get arrays of phases with the same times and check		*/
	/* each in case there are different phase codes present.	*/
	/* If found, all phase codes will be set the same.			*/
	for (i=0; i<n; i++){

		if (done[i])
			continue;

		j=0;				/* Number of phases with this time. */
		for (k=i; k<n; k++)
			if (p[samesta[k]].time==p[samesta[i]].time){
				sametime[j++] = samesta[k];
				done[k]=1;
			}

		/* Check array for entries with different phase codes.	*/
		/* and reset all but one if necessary.					*/
		if (same_time(sametime,j,sp,p) )
			return 1;
	}

	/* Go on to treat phases with the same phase code. */

	/* Array of flags parallel to samesta array to show that	*/
	/* that phase's phase code has already been dealt with.		*/
	for (i=0; i<n; i++)
		done[i]=0;

	/* Find phases for this station that have the same ISC phase code	*/
	/* and pass an array of their indexes on to the next stage.			*/
	for (i=0; i<n; i++){

		if (done[i])
			continue;

		j=0;				/* Number of phases with this phase code. */
		for (k=i; k<n; k++)
			if (strcmp(p[samesta[k]].phase,p[samesta[i]].phase)==0){
				samepha[j++] = samesta[k];
				done[k]=1;
			}

		/* Sort out the duplicates. */
		if (same_pha(samepha,j,p) )
			return 1;
	}
	return 0;
}

/*#DOC  Title:																*/
/*#DOC    same_pha    ( called only from same_sta() )						*/
/*#DOC  Desc:																*/
/*#DOC    Resolve duplicate phases.											*/
/*#DOC  Input Arguments:													*/
/*#DOC    samepha - array of p[] indexes for one reading and phase code.	*/
/*#DOC    n  - size of samepha array.										*/
/*#DOC    p  - array of pointers to structures containing phases.			*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/

/* Receives an array of phase structure indexes for a single		*/
/* station and phase code (samepha) and the size of this array (n).	*/
/* First checks whether more than one time has been reported for	*/
/* this duplicate. If so, weight all phases down depending on the 	*/
/* number of times reported. e.g. weight * 1/2 if there are two 	*/
/* arrival times.													*/
/* Then creates an index array for each time and weights all but	*/
/* one with weight_fact 0, effectively removing them from solution.	*/

int same_pha (int samepha[], int n, struct pha_rec p[])
{

	int sametime[PHA_PER_READ];
	int done[PHA_PER_READ];
	double unique_time[PHA_PER_READ];
	int number_of_times;
	int match;
	int i,j,k;
	int x;		/* Index of prefered phase in phase structure array */

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	if (diagnostic){
		fprintf(logfp,"same_pha: %s %s ",p[samepha[0]].phase,p[samepha[0]].sta);
		fprintf(logfp,"%d phases\n",n);
	}

	/* Find out how many different times were given for this phase. */
	unique_time[0]=p[samepha[0]].time;
	number_of_times=1;
	for (i=1; i<n; i++){
		match=0;
		for(j=0; j<number_of_times; j++){
			if (p[samepha[i]].time == unique_time[j]){
				match=1;
				break;
			}
		}
		if (!match)
			unique_time[number_of_times++] = p[samepha[i]].time;
	}
	if (diagnostic) printf ("%d unique times\n",number_of_times);

	/* Downweight phases if necessary. */
	if (number_of_times > 1)
		for (i=0; i<n; i++)
			p[samepha[i]].weight_factor /= number_of_times;

	/* Array of flags parallel to samepha array to show that	*/
	/* that phase's time has already been dealt with.			*/
	for (i=0; i<n; i++)
		done[i]=0;

	/* Find duplicates that have the same time and	*/
	/* weight out all but one of them.				*/
	for (i=0; i<n; i++){

		if (done[i])
			continue;

		j=0;				/* Number of phases with this time. */
		for (k=i; k<n; k++)
			if (p[samepha[k]].time == p[samepha[i]].time){
				sametime[j++] = samepha[k];
				done[k]=1;
			}

		/* Decide on prefered phase. 					*/
		/* Just take first one where pref_rd=rdid. 		*/
		for (k=0; k<j; k++)
			if (p[sametime[k]].rdid == p[sametime[k]].pref_rd){
				x = sametime[k];	/* Index of prefered phase in phase array */
				break;
			}

		/* If this phase doesn't occur in prefered reading */
		/* then take first example. */
		if (k==j){
			x = sametime[0];
		}

		/* Go through non-prefered ones setting weight_factor to 0. */
		for (k=0; k<j; k++){
			if (sametime[k] == x)
				continue;

			/* Remove duplicate from the solution. */
			/* And mark for possible updating of database. */
			p[sametime[k]].weight_factor = 0;
			p[sametime[k]].duplicate = 1;
			p[sametime[k]].pref_rd = p[x].rdid;
			if (diagnostic)
				printf ("%s %s DUP\n",p[sametime[k]].sta,p[sametime[k]].phase);

		}
	}
	return 0;
}

/*#DOC  Title:																*/
/*#DOC    same_time    ( called only from same_sta() )						*/
/*#DOC  Desc:																*/
/*#DOC    Ensures that phases with the same arrival time have the same code.*/
/*#DOC  Input Arguments:													*/
/*#DOC    sametime - array of p[] indexes for one reading and time.			*/
/*#DOC    n  - size of sametime array.										*/
/*#DOC    sp - pointer to structure containing current solution.			*/
/*#DOC    p  - array of pointers to structures containing phases.			*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/
/*#DOC  Calls:																*/
/*#DOC	  read_ttime() from read_ttime.c									*/

/* Receives an array of phase structure indexes for a single		*/
/* station and time (sametime) and the size of this array (n).		*/
/* Counts the different phase codes given. If it finds more than	*/
/* one code then residuals are calculated wrt the current solution	*/
/* (pointed to by sp) and all phase codes set to that which has		*/
/* the lowest residual.												*/

int same_time (int sametime[], int n, struct sol_rec* sp, struct pha_rec p[])
{
	int match;
	int number_of_codes;
	int phacode[PHA_PER_READ];
	double ttime[PHA_PER_READ];
	double obtime;
	double resid, min_resid;
	int min_resid_index;
	int i,j;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	if (diagnostic){
		fprintf(logfp,"same_time: ");
		fprintf(logfp,"%s %f ",p[sametime[0]].sta,p[sametime[0]].time);
		fprintf(logfp,"%d phases\n",n);
	}

	/* Get all the different phase codes given for this time. 				 */
	/* pP and sP are special and can co-exist with other codes at same time. */
	phacode[0]=sametime[0];
	number_of_codes=1;
	for (i=1; i<n; i++){

		if (strcmp(p[sametime[i]].phase,"pP")==0)
			continue;

		if (strcmp(p[sametime[i]].phase,"sP")==0)
			continue;

		match=0;
		for(j=0; j<number_of_codes; j++){
			if (strcmp(p[sametime[i]].phase,p[phacode[j]].phase)==0){
				match=1;
				break;
			}
		}
		if (!match)
			phacode[number_of_codes++]=sametime[i];
	}
	if (diagnostic)
		printf ("same_time: %d different phases\n",number_of_codes);

	/* Almost always, there will only be one phase code. */
	if (number_of_codes == 1)
		return 0;

	/* Calculate observed time for all these phases. */
	obtime = p[sametime[0]].time - sp->time;

	/* Loop over phase codes to find smallest residual. */
	min_resid=fabs(NULLVAL);
	for (i=0; i<number_of_codes; i++){

		if( read_ttime(sp, &p[phacode[i]]) ){
			p[phacode[i]].ttime = NULLVAL;
		}

		resid = fabs(p[phacode[i]].ttime - obtime);
		if (resid < min_resid){
			min_resid = resid;
			min_resid_index = phacode[i];
		}
		if (diagnostic){
			fprintf(logfp,"same_time: %s %f ",p[phacode[i]].phase,obtime);
			fprintf(logfp,"%f %f\n",p[phacode[i]].ttime,resid);
		}
	}
	if (diagnostic)
		fprintf(logfp,"same_time: set to %s\n",p[min_resid_index].phase);

	/* Reset phase_codes to that which has the smallest residual. */
	for (i=0; i<n; i++)
		strcpy(p[sametime[i]].phase,p[min_resid_index].phase);

	return 0;
}
