#include "location.h"

/*#DOC  Title:																*/
/*#DOC    purge_pha															*/
/*#DOC  Desc:																*/
/*#DOC    Sets 'purged' field of a phase if |resid| > purge_resid.			*/
/*#DOC  Input Arguments:													*/
/*#DOC    ep  - pointer to structure containing event information.			*/
/*#DOC    sp  - pointer to structure containing current solution.			*/
/*#DOC    p[] - array of phase structures.									*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/
/*#DOC  Uses:																*/
/*#DOC    print_pha() from utils.c											*/

/* Checks if phases have been purged yet and if not then does so.			*/
/* Threshold used is in config.txt.											*/
/* If purging is switched off on instruction line then init_sol sets		*/
/* phases_purged=1 at the outset so that this function never runs.			*/
/* Resets the converged flag so that iterating will continue. 				*/

/* Purged flag in phase structure can have three values:					*/
/*   0 Still available.														*/
/*   1 Phase purged but other good phases in same reading.					*/
/*   2 Phase purged along with all other phases in this reading.			*/

int purge_pha (struct event_rec *ep, struct sol_rec *sp,  struct pha_rec p[])
{
	extern double purge_resid;		/* From config file */

	int prev_rdid;
	int reading[PHA_PER_READ];
	int keep,purged;
	int i,j,k;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* If phases have already been purged then don't do it again. 			*/
	/* This is also how purging is switched off - phases_purged starts as 1.*/
	if (sp->phases_purged)
		return 0;

	/* Purge large resids but not those set to NULLVAL. */
	/* If no other phases in this reading have small residuals then 	*/
	/* purge all the phases (to stop some being used for magnitudes.) 	*/
	prev_rdid = 0;
	j = 0;					/* Number of phases for a reading. 		*/
	keep = 1;				/* Default is don't purge whole reading.*/
	purged = 0;				/* Count of purges made.				*/
	for (i=0; i<sp->numphas; i++){

		/* Deal with previous reading in case been marked bad. */
		if (p[i].rdid != prev_rdid){
			if (!keep){
				/* Check through bad reading for good phases. */
				for (k=reading[0]; k<=reading[j-1]; k++)
					if (fabs(p[k].resid) <= purge_resid)
						keep = 1;

				/* Reading still bad. */
				if (!keep)
					for (k=reading[0]; k<=reading[j-1]; k++)
						p[k].purged = 2;

				keep = 1;
			}
			j=0;
			prev_rdid = p[i].rdid;
		}

		/* Check current phase. */
		if (fabs(p[i].resid) > purge_resid && p[i].resid != NULLVAL){
			p[i].purged = 1;
			keep = 0;
			purged++;
		}
		reading[j++] = i;

		/* Check not going past end of reading array. */
		if (j>PHA_PER_READ){
			sprintf(errstr,"purge_pha: %s: too many phases",p[i].sta);
			return 1;
		}
	}

	if (diagnostic){
		fprintf(logfp,"\nphases with |resid| > %f purged:\n",purge_resid);
		fprintf(logfp,"(weights not reset yet)\n");
		print_pha(sp->numphas, p);
	}

	/* Set flag in solution structure so that purging only happens once. */
	sp->phases_purged = 1;

	/* Reset the converged flag so that iterating will continue.	*/
	/* Only do this if at least one phase has been purged.			*/
	if (purged > 0)
		sp->converged = 0;

	return 0;
}
