#include "location.h"

#define MAXOPT 10	/* maximum number of possible options. */

/*#DOC  Title:																*/
/*#DOC    read_instruction													*/
/*#DOC  Desc:																*/
/*#DOC    Parses a line of instructions for evid and options.				*/
/*#DOC  Input Arguments:													*/
/*#DOC    instruction  - string containing a single line of instructions.	*/
/*#DOC    ep - pointer to structure containing event information.			*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/

/* Location subroutine to read a line of instructions.					*/
/* Normally this line will have come from STDIN.						*/
/* One or more options may be given in any order in parameter=value 	*/
/* format seperated by white space.										*/
/* Possible options:													*/
/*					seed  (an agency code)								*/
/*					depth (an agency code or a number)					*/
/*					location (an agency code)							*/
/*					lat,lon (numbers - must have both)					*/
/*					time (an agency code or a number)					*/
/*					no_reid_phase 	(presence sets flag to 0)			*/
/*					no_purge_phase 	(presence sets flag to 0)			*/
/*					fix_on_depdp 	(presence sets flag to 1)			*/
/*					filename (read next isf event from file)			*/

/* evid and options are stored in the event structure passed from main.	*/

/* fix_on_depdp option can be used with another fixed depth option.		*/
/* Fixes depth to be used in initial depdp calculation - the depdp		*/
/* will then be used in the solution.									*/

int read_instruction (char* instruction,struct event_rec *ep,char *infile_name)
{
	char evid[VALLEN];
	char opt[MAXOPT][PARLEN];
	char val[MAXOPT][VALLEN];
	char *remnant;
	int i,j,k;
	int value_option;
	char c;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* Set default values for options. */
	ep->evid = NULLVAL;
	ep->seed_agency[0]			= '\0';
	ep->depth_agency[0]			= '\0';
	ep->location_agency[0]		= '\0';
	ep->time_agency[0]			= '\0';
	ep->fixed_depth				= NULLVAL;
	ep->fixed_lat				= NULLVAL;
	ep->fixed_lon				= NULLVAL;
	ep->fixed_time				= NULLVAL;
	ep->reid_phase				= 1;
	ep->purge_phase				= 1;
	ep->fix_on_depdp			= 0;
	infile_name[0]				= '\0';

	if (diagnostic)
		fprintf(logfp,"read_instruction: %s",instruction);

	/* Remove leading white space */
	i=0;
	while (instruction[i] == ' ' || instruction[i] == '\t'){i++;}


	/* Read in unknown number of options. */
	k=0;
	while (instruction[i] != '\n'){

		/* Option name */
		j=0;
		value_option = 1;
		while (instruction[i] != '\n'){
			if(instruction[i] == '='){
				i++;
				break;
			}
			if(instruction[i] == ' '){
				value_option = 0;
				i++;
				break;
			}
			opt[k][j++]=instruction[i++];
		}
		opt[k][j]='\0';

		/* Option value */
		j=0;
		if (value_option)
			while (instruction[i] != '\n'){
 				if (instruction[i] == ' ' ){
					i++;
					break;
				}
				val[k][j++]=instruction[i++];
			}
		val[k][j]='\0';
		k++;
	}

	/* Check any options given and update event structure. */
	for (i=0; i<k; i++){

		/* Each option can only be given once. */
		for (j=0; j<i; j++)
			if (strcmp(opt[i],opt[j])==0){
				sprintf(errstr,"read_instruction: two option %s",opt[i]);
				return 1;
			}

		/* Number on its own interpreted as evid. */
		if (isdigit(opt[i][0])){
			if (ep->evid != NULLVAL || infile_name[0]){
				sprintf(errstr,"read_instruction: two evids %s",instruction);
				return 1;
			}
			ep->evid = (int)strtod(opt[i],&remnant);
			if (remnant[0]){
				sprintf(errstr,"read_instruction: bad evid %s",opt[i]);
				return 1;
			}
		}
		else if (strcmp(opt[i],"seed")==0){
			strcpy(ep->seed_agency,val[i]);
		}
		else if (strcmp(opt[i],"depth")==0){
			if (isdigit(val[i][0])){
				ep->fixed_depth = strtod(val[i],&remnant);
				if (remnant[0]){
					sprintf(errstr,"read_instruction: bad depth %s",val[i]);
					return 1;
				}
			}
			else
				strcpy(ep->depth_agency,val[i]);
		}
		else if (strcmp(opt[i],"location")==0){
			strcpy(ep->location_agency,val[i]);
		}
		else if (strcmp(opt[i],"lat")==0){
			ep->fixed_lat = strtod(val[i],&remnant);
			if (remnant[0]){
				sprintf(errstr,"read_instruction: bad lat %s",val[i]);
				return 1;
			}
		}
		else if (strcmp(opt[i],"lon")==0){
			ep->fixed_lon = strtod(val[i],&remnant);
			if (remnant[0]){
				sprintf(errstr,"read_instruction: bad lon %s",val[i]);
				return 1;
			}
		}
		else if (strcmp(opt[i],"time")==0){
			if (isdigit(val[i][0])){
				if (ep->fixed_time = read_time(val[i]) == 0){
					sprintf(errstr,"read_instruction: bad time %s",val[i]);
					return 1;
				}
			}	
			else
				strcpy(ep->time_agency,val[i]);
		}
		else if (strcmp(opt[i],"no_reid_phase")==0){
			ep->reid_phase = 0;
		}
		else if (strcmp(opt[i],"no_purge_phase")==0){
			ep->purge_phase = 0;
		}
		else if (strcmp(opt[i],"fix_on_depdp")==0){
			ep->fix_on_depdp = 1;
		}
		else if (strcmp(opt[i],"filename")==0){
			if (ep->evid != NULLVAL){
				sprintf(errstr,"read_instruction: two evids %s",instruction);
				return 1;
			}
			strcpy(infile_name,val[i]);
		}
		else{
			sprintf(errstr,"read_instruction: unknown option %s",opt[i]);
			return 1;
		}
		if (diagnostic)
			fprintf(logfp,"read_instruction:%d %s %s\n",i,opt[i],val[i]);
	}

	/* Must be an evid (or filename) somewhere on line. */
	if (ep->evid == NULLVAL && !infile_name[0]){
		sprintf(errstr,"read_instruction: no evid %s",instruction);
		return 1;
	}

	/* Checks to see that not fixing something two ways at once. */
	if (ep->fixed_lat != NULLVAL || ep->fixed_lon != NULLVAL){

		if (ep->fixed_lat == NULLVAL || ep->fixed_lon == NULLVAL){
			sprintf(errstr,"read_instruction: must fix both lat,lon");
			return 1;			
		}

		if (ep->location_agency[0]){
			sprintf(errstr,"read_instruction: location fixed twice");
			return 1;			
		}
	}
	if (ep->location_agency[0]
	&& (ep->fixed_depth != NULLVAL || ep->depth_agency[0])){
		sprintf(errstr,"read_instruction: depth fixed twice");
		return 1;			
	}

	return 0;
}
