#include "location.h"
#include "isf_bul.h"

#define MAXSTA 10000

struct sta_rec {
	char net[NETLEN];
	char sta[STALEN];
	double lat;
	double lon;
	double elev;
};

struct sta_rec stalist[MAXSTA];
int numsta;

int read_stafile(struct sta_rec stalist[]);

/*#DOC  Title:																*/
/*#DOC    read_isf_event													*/
/*#DOC  Desc:																*/
/*#DOC    Reads next event from an ISF bulletin format file.				*/
/*#DOC  Input Arguments:													*/
/*#DOC    infile  - file pointer to read from.								*/
/*#DOC    ep - pointer to structure containing event information.			*/
/*#DOC    pp - pointer to array of pointers to phase structures.			*/
/*#DOC    hp - pointer to array of pointers to hypocentre structures.		*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/
/*#DOC  Uses:																*/
/*#DOC	  read_stafile()													*/
/*#DOC	  join_time() from utils.c.											*/
/*#DOC	  print_pha() from utils.c.											*/
/*#DOC	  functions from isf_read.c.										*/

int read_isf_event(FILE *infile, struct event_rec *ep, 
									struct hyp_rec *hp[], struct pha_rec *pp[])
{
	struct hyp_rec *h;
	struct pha_rec *p;
    char line[ISF_LINE_LEN*2];
	int yyyy,mm,dd,hh,mi,ss,msec;
	char evid[ISF_EVID_LEN+1];
	char region[ISF_REGION_LEN+1];
	char comment[ISF_LINE_LEN+1];
	char prev_sta[STALEN];
	struct pha_rec temp_pha;
	int rdid;
	int found;
	int i,j,k;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* Arrays for storing hypocentre values as they are read in. */
	/* Non-array variables used for unwanted values. */
	double hyp_time[MAXHYP];
	char timfix[MAXHYP];
	float stime[MAXHYP];
	float sdobs[MAXHYP];
	float lat[MAXHYP];
	float lon[MAXHYP];
	char epifix[MAXHYP];
	float majax[MAXHYP];
	float minax[MAXHYP];
	int theta[MAXHYP];
	float depth[MAXHYP];
	char depfix[MAXHYP];
	float sdepth[MAXHYP];
	int ndef[MAXHYP];
	int nsta[MAXHYP];
	int azimgap[MAXHYP];
	float mindist[MAXHYP];
	float maxdist[MAXHYP];
	char antype;
	char loctype;
	char etype[MAXHYP][ISF_ETYPE_LEN+1];
	char author[MAXHYP][ISF_AUTHOR_LEN+1];
	char origid[MAXHYP][ISF_ORIGID_LEN+1];


	/* Arrays for reading in phase data. */
	/* Non-array variables used for unwanted values. */
	char sta[MAXPHA][ISF_STA_LEN+1];
	float delta[MAXPHA];
	float esaz;
	char phase[MAXPHA][ISF_PHASE_LEN+1];
	double pha_time[MAXPHA];
	float azim[MAXPHA],slow[MAXPHA];
	float timeres,azimres,slowres;
	char timedef,azimdef,slowdef;
	float snr;
	float amp[MAXPHA],per[MAXPHA];
	char picktype;
	char sp_fm[MAXPHA];
	char detchar[MAXPHA];
	char magtype[ISF_MAGTYPE_LEN+1];
	char magind;
	float mag;
	char arrid[MAXPHA][ISF_ARRID_LEN+1];


	/* If first time here then read in ISF station information from file. */
	if (!stalist[0].sta[0]){
		if ((numsta = read_stafile(stalist))==0){
			sprintf(errstr,"read_stafile: no stations read");
			return 1;
		}
		if (diagnostic)
			fprintf(logfp,"read_isf_event: read in station file.\n");
	}

	/* Start parsing report. */

	/* Skip lines until reach an event id line. */
	while (fgets(line,ISF_LINE_LEN*2,infile))
		if (read_event_id(line,evid,region)==0)
			break;
	if (strcmp(evid,""))
		ep->evid = atoi(evid);
	else{
		sprintf(errstr,"read_isf_event: no event line found");
		return 1;
	}

	if (diagnostic)
		fprintf(logfp,"read_isf_event: found event id %d\n",ep->evid);

	/* Skip origin header. */
	fgets(line,ISF_LINE_LEN*2,infile);
	if (read_origin_head(line)){
		sprintf(errstr,"read_isf_event: origin_head: %s",isf_bulletin_error);
		return 1;
	}

	/* Loop over origin block. */
	i=0;
	while (fgets(line,ISF_LINE_LEN*2,infile)){

		if (all_blank(line))
			break;

		if (read_comment(line,comment) == 0)
			continue;

		if (read_origin(line,&yyyy,&mm,&dd,&hh,&mi,&ss,&msec,&timfix[i],
			&stime[i],&sdobs[i],&lat[i],&lon[i],&epifix[i],
			&majax[i],&minax[i],&theta[i],&depth[i],&depfix[i],
			&sdepth[i],&ndef[i],&nsta[i],&azimgap[i],&mindist[i],
			&maxdist[i],&antype,&loctype,etype[i],author[i],origid[i]) == 0){

			if (msec == ISF_NULL) msec = NULLVAL;
			hyp_time[i++] = join_time(yyyy,mm,dd,hh,mi,ss,msec);
		}
		else{
			sprintf(errstr,"read_isf_event: origin: %s",isf_bulletin_error);
			handle_error();
		}
	}
	if ((ep->numhyps=i)==0){
		sprintf(errstr,"read_isf_event: no hypocentres read");
		return 1;
	}
	if (diagnostic)
		fprintf(logfp,"read_isf_event: %d hypocentres read\n",ep->numhyps);

	/* Skip lines until reach a phase header. */
	while (fgets(line,ISF_LINE_LEN*2,infile))
		if (read_phase_head(line)==0){
			found=1;
			break;
		}
	if (!found){
		sprintf(errstr,"read_isf_event: no phase header found");
		return 1;
	}

	/* Loop over phase block. */
	i=0;
	while (fgets(line,ISF_LINE_LEN*2,infile)){

		if (all_blank(line))
			break;

		if (read_comment(line,comment) == 0)
			continue;

		if (read_phase(line,sta[i],&delta[i],&esaz,phase[i],&hh,&mi,&ss,&msec,
			&timeres,&azim[i],&azimres,&slow[i],&slowres,&timedef,
			&azimdef,&slowdef,&snr,&amp[i],&per[i],&picktype,&sp_fm[i],
			&detchar[i],magtype,&magind,&mag,arrid[i]) == 0){

			/* Take yyyy,mm,dd from last hyp - then check. */
			if (hh == ISF_NULL)
				pha_time[i] = NULLVAL;
			else{
				if (msec == ISF_NULL) msec = NULLVAL;
				pha_time[i] = join_time(yyyy,mm,dd,hh,mi,ss,msec);

				/* Only way phase could be before origin is if day	*/
				/* changes - as being turned into number of secs	*/
				/* since epoch month changes unimportant. 			*/
				if (pha_time[i] < hyp_time[0])
					pha_time[i] = join_time(yyyy,mm,dd+1,hh,mi,ss,msec);
			}
			if (delta[i] == ISF_NULL){
				sprintf(errstr,"read_isf_event: phases must have delta");
				return 1;
			}
			i++;
		}
		else{
			sprintf(errstr,"read_isf_event: phase: %s",isf_bulletin_error);
			handle_error();
		}
	}
	if ((ep->numphas=i)==0){
		sprintf(errstr,"read_isf_event: no phases read");
		return 1;
	}
	if (diagnostic)
		fprintf(logfp,"read_isf_event: %d phases read\n",ep->numphas);

	/* Assign memory to structure arrays (via pointers so can pass back). */
	*hp = (struct hyp_rec *) calloc (ep->numhyps, sizeof(struct hyp_rec));
	if (*hp == NULL){
		sprintf(errstr,"read_isf_event: hyp memory problem.");
		return 1;
	}
	*pp = (struct pha_rec *) calloc (ep->numphas, sizeof(struct pha_rec));
	if (*pp == NULL){
		sprintf(errstr,"read_isf_event: pha memory problem.");
		free(*hp);
		return 1;
	}

	/* Fill in the hypocentre structures. */
	h = *hp;
	for (i=0; i< ep->numhyps; i++){

		h[i].hypid = atoi(origid[i]);
		h[i].time  = hyp_time[i];
		h[i].lat   = lat[i];
		h[i].lon   = lon[i];
		calc_dircos(lat[i],lon[i],h[i].dircos);

		if (depth[i] == ISF_NULL)
			h[i].depth = NULLVAL;
		else
			h[i].depth = depth[i];
		if (ndef[i] == ISF_NULL)
			h[i].ndef = NULLVAL;
		else
			h[i].ndef = ndef[i];
		if (nsta[i] == ISF_NULL)
			h[i].ndefsta = NULLVAL;
		else
			h[i].ndefsta = nsta[i];
		h[i].nsta = NULLVAL;

		if (mindist[i] == ISF_NULL)
			h[i].mindist = NULLVAL;
		else
			h[i].mindist = mindist[i];
		if (maxdist[i] == ISF_NULL)
			h[i].maxdist = NULLVAL;
		else
			h[i].maxdist = maxdist[i];
		if (azimgap[i] == ISF_NULL)
			h[i].azimgap = NULLVAL;
		else
			h[i].azimgap = azimgap[i];

		strcpy(h[i].etype,etype[i]);
		strcpy(h[i].agency,author[i]);

		if (sdobs[i] == ISF_NULL)
			h[i].sdobs = NULLVAL;
		else
			h[i].sdobs = sdobs[i];
		if (stime[i] == ISF_NULL)
			h[i].stime = NULLVAL;
		else
			h[i].stime = stime[i];
		if (sdepth[i] == ISF_NULL)
			h[i].sdepth = NULLVAL;
		else
			h[i].sdepth = sdepth[i];
		if (minax[i] == ISF_NULL)
			h[i].minax = NULLVAL;
		else
			h[i].minax = minax[i];
		if (majax[i] == ISF_NULL)
			h[i].majax = NULLVAL;
		else
			h[i].majax = majax[i];
		if (theta[i] == ISF_NULL)
			h[i].theta = NULLVAL;
		else
			h[i].theta = theta[i];

		h[i].depfix = h[i].epifix = h[i].timfix = 0;
		if (depfix[i]=='f')
			h[i].depfix = 1;
		if (epifix[i]=='f')
			h[i].epifix = 1;
		if (timfix[i]=='f')
			h[i].timfix = 1;
	}

	/* Fill in the phase structures. */
	p = *pp;
	for (i=0; i< ep->numphas; i++){

		/* Associations all to top hypocentre. */
		p[i].hypid = h[0].hypid;
		p[i].phid  = atoi(arrid[i]);

		strcpy(p[i].rep_phase,phase[i]);
		strcpy(p[i].sta,sta[i]);

		if (pha_time[i] == ISF_NULL)
			p[i].time = NULLVAL;
		else
			p[i].time = pha_time[i];

		if (slow[i] == ISF_NULL)
			p[i].slow = NULLVAL;
		else
			p[i].slow = slow[i];
		if (azim[i] == ISF_NULL)
			p[i].azim = NULLVAL;
		else
			p[i].azim = azim[i];

		p[i].delta = delta[i];

		if (amp[i] == ISF_NULL)
			p[i].numamps = 0;
		else{
			p[i].numamps = 1;
			p[i].a[0].amp = amp[i];
			p[i].a[0].per = per[i];
		}
		p[i].sp_fm=sp_fm[i];

		p[i].detchar=detchar[i];

	}

	/* Sort phase structures by time then by sta/delta. */
	for (i=0; i < ep->numphas; i++)
		for (j=i-1; j > -1; j--)
			if ((p[j].time > p[j+1].time && p[j+1].time != NULLVAL)
			|| p[j].time == NULLVAL){
				temp_pha = p[j];
				p[j]     = p[j+1];
				p[j+1]   = temp_pha;
			}

	for (i=0; i < ep->numphas; i++)
		for (j=i-1; j > -1; j--)
			if (strcmp(p[j].sta,p[j+1].sta)>0){
				temp_pha = p[j];
				p[j]     = p[j+1];
				p[j+1]   = temp_pha;
			}

	for (i=0; i < ep->numphas; i++)
		for (j=i-1; j > -1; j--)
			if (p[j].delta > p[j+1].delta){
				temp_pha = p[j];
				p[j]     = p[j+1];
				p[j+1]   = temp_pha;
			}

	/* Split phases into readings and use station array to assign lat,lon. */
	prev_sta[0] = '\0';
	rdid = 0;
	for (i=0; i < ep->numphas; i++){
		if (strcmp(p[i].sta,prev_sta)){
			p[i].rdid = rdid++;
			p[i].init = 1;

			for (j=0; j<numsta; j++)
				if (strcmp(p[i].sta,stalist[j].sta)==0){
					p[i].sta_lat  = stalist[j].lat;
					p[i].sta_lon  = stalist[j].lon;
					p[i].sta_elev = stalist[j].elev;
					calc_dircos(p[i].sta_lat,p[i].sta_lon,p[i].dircos);
					break;
				}
			if (j==numsta){
				sprintf(errstr,"read_isf_event: sta %s not found.",p[i].sta);
				handle_error();
				continue;
			}
			strcpy(prev_sta,p[i].sta);
		}
		else{
			p[i].rdid     = p[i-1].rdid;
			p[i].sta_lat  = p[i-1].sta_lat;
			p[i].sta_lon  = p[i-1].sta_lon;
			p[i].sta_elev = p[i-1].sta_elev;
			for (k=1; k<=6; k++)
				p[i].dircos[k] = p[i-1].dircos[k];
		}
		p[i].pref_rd = p[i].rdid;
	}

	if (diagnostic){
		fprintf(logfp,"read_isf_event:\n");
		print_pha(ep->numphas, p);
	}
	return 0;
}
