#include "location.h"
#include "jb_model.h"
#include "jb_tables.h"

/*#DOC  Title:																*/
/*#DOC    read_jb.c															*/
/*#DOC  Desc:																*/
/*#DOC   File with functions that need to read the JB tables.				*/
/*#DOC   The two functions called from outside this file are:				*/
/*#DOC    read_jb															*/
/*#DOC    jb_pP_P															*/


/*#DOC  Title:																*/
/*#DOC    read_jb															*/
/*#DOC  Desc:																*/
/*#DOC    Fills 'ttime, 'dtdh', and 'dtdd' fields for a phase.				*/
/*#DOC  Input Arguments:													*/
/*#DOC    depth - depth of current solution.								*/
/*#DOC    pp    - pointer to structure containing a phase.					*/
/*#DOC  Return:																*/
/*#DOC    0/1/2/3 for success/failure/warn/reid phase.						*/
/*#DOC  Calls:																*/
/*#DOC    Functions from this file for reading tables.						*/

/* For P/PKP or S/SKS 3 possible cases: 								*/
/* If edep < 0.016 earth radii and delta < 1.3 							*/
/*		then use 'Short Epicentral Distances' table.					*/
/* If edep < 0 earth radii (delta > 8 or would be assuming Pg, Pn etc.) */
/* 		then use first column of standard tables but interpolate and 	*/
/* 		calculate derivatives using velocities.							*/
/* Otherwise															*/
/*		use standard tables												*/

/* Save dtdh in s/km rather than s/e.r.*100.							*/

int read_jb (double depth, struct pha_rec *pp)
{
	int status;
	double ttime;
	double dtdd;
	double dtdh;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* Call subroutine relevant for phase code. */
	status = 0;

	/* Pg */
	if (strcmp(pp->phase,"Pg") == 0)
		status = ps_g('P',pp->delta,depth,&ttime,&dtdh,&dtdd);
	/* Pb */
	else if (strcmp(pp->phase,"Pb") == 0)
		status = ps_star('P',pp->delta,depth,&ttime,&dtdh,&dtdd);
	/* Pn */
	else if (strcmp(pp->phase,"Pn") == 0)
		status = ps_crust('P',pp->delta,depth,&ttime,&dtdh,&dtdd);
	/* Sg */
	else if (strcmp(pp->phase,"Sg") == 0)
		status = ps_g('S',pp->delta,depth,&ttime,&dtdh,&dtdd);
	/* Sb */
	else if (strcmp(pp->phase,"Sb") == 0)
		status = ps_star('S',pp->delta,depth,&ttime,&dtdh,&dtdd);
	/* Sn */
	else if (strcmp(pp->phase,"Sn") == 0)
		status = ps_crust('S',pp->delta,depth,&ttime,&dtdh,&dtdd);

	/* P - subroutine chosen according to delta and source depth. */
	else if (strcmp(pp->phase,"P") == 0){

		if (diagnostic)
			fprintf(logfp,"read_jb: P delta=%f depth=%f\n",pp->delta,depth);

		/* Station very near to source, depth between MOHO and SHORT_DEPTH. */
		if (pp->delta < SHORT_DELTA && depth < SHORT_DEPTH)
			status = ps_short(&p_short,pp->delta,depth,&ttime,&dtdh,&dtdd);

		/* Shallow source with station beyond crustal phase distances. */
		else if (depth < MOHO)
			status = ps_crust('P',pp->delta,depth,&ttime,&dtdh,&dtdd);

		/* P default. */
		else
			status = ps(&p_tab,pp->delta,depth,&ttime,&dtdh,&dtdd);
	}
	/* PKP - same subroutine as default P but different table. */
	else if (strcmp(pp->phase,"PKP") == 0)
		status = ps(&pkp_tab,pp->delta,depth,&ttime,&dtdh,&dtdd);

	/* Pdiff - no derivatives returned, no use in solution. */
	else if (strcmp(pp->phase,"Pdiff") == 0){
		status = pdiff(&p_tab,pp->delta,depth,&ttime);
		dtdh = NULLVAL;
		dtdd = NULLVAL;
	}

	/* S - subroutine chosen according to delta and source depth. */
	else if (strcmp(pp->phase,"S") == 0){

		if (diagnostic)
			fprintf(logfp,"read_jb: S delta=%f depth=%f\n",pp->delta,depth);

		/* Station very near to source, depth between MOHO and SHORT_DEPTH. */
		if (pp->delta < SHORT_DELTA && depth < SHORT_DEPTH)
			status = ps_short(&s_short,pp->delta,depth,&ttime,&dtdh,&dtdd);

		/* Shallow source with station beyond crustal phase distances. */
		else if (depth < MOHO)
			status = ps_crust('S',pp->delta,depth,&ttime,&dtdh,&dtdd);

		/* S default. */
		else
			status = ps(&s_tab,pp->delta,depth,&ttime,&dtdh,&dtdd);
	}
	/* SKS - same subroutine as default S but different table. */
	else if (strcmp(pp->phase,"SKS") == 0)
		status = ps(&sks_tab,pp->delta,depth,&ttime,&dtdh,&dtdd);

	/* From here phases are dealt with in other_phase,	*/
	/* which does not return derivatives.				*/
	/* PcP */
	else if (strcmp(pp->phase,"PcP") == 0){
		status = other_phase(&pcp_surf_tab,pp->delta,depth,&ttime);
		dtdh = NULLVAL;
		dtdd = NULLVAL;
	}
	/* ScS */
	else if (strcmp(pp->phase,"ScS") == 0){
		status = other_phase(&scs_surf_tab,pp->delta,depth,&ttime);
		dtdh = NULLVAL;
		dtdd = NULLVAL;
	}
	/* PP */
	else if (strcmp(pp->phase,"PP") == 0){
		status = other_phase(&pp_surf_tab,pp->delta,depth,&ttime);
		dtdh = NULLVAL;
		dtdd = NULLVAL;
	}
	/* PPP */
	else if (strcmp(pp->phase,"PPP") == 0){
		status = other_phase(&ppp_surf_tab,pp->delta,depth,&ttime);
		dtdh = NULLVAL;
		dtdd = NULLVAL;
	}
	/* PS */
	else if (strcmp(pp->phase,"PS") == 0){
		status = other_phase(&ps_surf_tab,pp->delta,depth,&ttime);
		dtdh = NULLVAL;
		dtdd = NULLVAL;
	}
	/* SP */
	else if (strcmp(pp->phase,"SP") == 0){
		status = other_phase(&sp_surf_tab,pp->delta,depth,&ttime);
		dtdh = NULLVAL;
		dtdd = NULLVAL;
	}
	/* SPP */
	else if (strcmp(pp->phase,"SPP") == 0){
		status = other_phase(&spp_surf_tab,pp->delta,depth,&ttime);
		dtdh = NULLVAL;
		dtdd = NULLVAL;
	}
	/* PSP / PPS*/
	else if (strcmp(pp->phase,"PSP") == 0 || strcmp(pp->phase,"PPS") == 0){
		status = other_phase(&pps_psp_surf_tab,pp->delta,depth,&ttime);
		dtdh = NULLVAL;
		dtdd = NULLVAL;
	}
	/* SS */
	else if (strcmp(pp->phase,"SS") == 0){
		status = other_phase(&ss_surf_tab,pp->delta,depth,&ttime);
		dtdh = NULLVAL;
		dtdd = NULLVAL;
	}
	/* PSS */
	else if (strcmp(pp->phase,"PSS") == 0){
		status = other_phase(&pss_surf_tab,pp->delta,depth,&ttime);
		dtdh = NULLVAL;
		dtdd = NULLVAL;
	}
	/* SSP / SPS*/
	else if (strcmp(pp->phase,"SSP") == 0 || strcmp(pp->phase,"SPS") == 0){
		status = other_phase(&ssp_sps_surf_tab,pp->delta,depth,&ttime);
		dtdh = NULLVAL;
		dtdd = NULLVAL;
	}
	/* SSS */
	else if (strcmp(pp->phase,"SSS") == 0){
		status = other_phase(&sss_surf_tab,pp->delta,depth,&ttime);
		dtdh = NULLVAL;
		dtdd = NULLVAL;
	}
	/* Unknown phase - should have been taken care of in id_pha. */
	else {
		sprintf(errstr,"%d: bad code: %s",pp->phid,pp->phase);
		return 1;
	}

	/* Record and return actual return status from function */
	/* as could be a warning or a serious error.			*/					
	if (status)
		return status;

	pp->ttime = ttime;
	pp->dtdh  = dtdh;
	pp->dtdd  = dtdd;

	return 0;
}

/*#DOC  Title:																*/
/*#DOC    ps																*/
/*#DOC  Desc:																*/
/*#DOC    Looks up given JB table and supplies ttime, dtdd, and dtdh.		*/
/*#DOC  Input Arguments:													*/
/*#DOC    tp    - pointer to structure with travel-time table.				*/
/*#DOC    delta - distance between phase and source in degrees.				*/
/*#DOC    depth - depth of current solution.								*/
/*#DOC    ttime - pointer to variable for travel time.						*/
/*#DOC    dtdh  - pointer to variable for dt/dh.							*/
/*#DOC    dtdd  - pointer to variable for dt/dd.							*/
/*#DOC  Return:																*/
/*#DOC    0/1/2 for success/failure/warn.									*/
/*#DOC  Calls:																*/
/*#DOC    fill_matrix() from this file.										*/
/*#DOC    interpol() and int_diff() from this file.							*/

/* Read the standard Jeffrey's and Bullen tables for P, PKP, S, and SKS.	*/
/* P and PKP are allowed below 105 and above 106 degrees respectively.		*/
/* (although in practice use Pdiff above 102)								*/
/* S and SKS overlap between 62 and 107 degrees.							*/

int ps( struct tab_rec *tp, double delta, double depth,
				double *ttime, double *dtdh, double *dtdd)
{                                              
	double depth_frac,delta_frac;
	double depth_erad;
	int matrix[3][3];
	double t[3],d[3];
	int col1,row1,depth_dir,delta_dir;
	int numval;
	int i;

	int fill_matrix (struct tab_rec*, int col1, int row1,
				 	 int depth_dir, int delta_dir, int matrix[][]);

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* Get column by using depth in earth radii * 100 */
	/* One out of step because 1st column is surface, 2nd column is 0.00 */
	depth_erad = (depth - MOHO)/MOHO_RADIUS;
	if (depth_erad < tp->min_depth || depth_erad > tp->max_depth){
		sprintf(errstr,"ps: depth out of range: %f",depth);
		return 1;
	}
	col1  = depth_erad*100 + 1; 			
	depth_frac = depth_erad*100 - (int)(depth_erad*100);

	/* Get row by using delta */
	if (delta < tp->min_delta || delta > tp->max_delta){
		sprintf(errstr,"ps: delta out of range: %f",delta);
		return 1;
	}
	row1 = delta - tp->min_delta;
	delta_frac = delta - (int)delta;

	if (diagnostic){
		fprintf(logfp,"ps: depth=%f col1=%d ",depth_erad,col1);
		fprintf(logfp,"depth_frac=%f\n",depth_frac);
		fprintf(logfp,"ps: delta=%f row1=%d ",delta,row1);
		fprintf(logfp,"delta_frac=%f\n",delta_frac);
	}

	/* Interpolate forwards in first half of table and backwards in second */
	/* half of table to avoid the ends. */
	if (delta > tp->switch_dir_delta){
		delta_dir = -1;
		row1++;
		delta_frac = 1 - delta_frac;
	}
	else
		delta_dir = 1;

	/* Fill a matrix with 9 values from the table */
	depth_dir = 1;
	numval = fill_matrix(tp,col1,row1,depth_dir,delta_dir,matrix);

	if (diagnostic){
		fprintf(logfp,"ps: %d %d %d\n",matrix[0][0],matrix[1][0],matrix[2][0]);
		fprintf(logfp,"    %d %d %d\n",matrix[0][1],matrix[1][1],matrix[2][1]);
		fprintf(logfp,"    %d %d %d\n",matrix[0][2],matrix[1][2],matrix[2][2]);
	}

	/* If at bottom of table fill in the missing value(s) */
	if ( numval==8 && matrix[2][0]==0){

		/* If at left hand end of table estimate value for matrix[2][0]. */
		if (col1 == 1){
			matrix[2][0]  = 2*(matrix[1][0] + matrix[2][1] + matrix[1][1]);
			matrix[2][0] -=  matrix[0][0] + matrix [2][2] + matrix[0][2];
			matrix[2][0] /= 3;
			numval = 9;
		}
		/* Otherwise invert depth order and try again */
		else {
			depth_dir = -1;
			col1 += 2;
			depth_frac = 1 - depth_frac;
			numval = fill_matrix(tp,col1,row1,depth_dir,delta_dir,matrix);
		}
	}
	else if (numval==7 && matrix[1][0]==0 && matrix[2][0]==0){

		if ( delta_dir== 1 && (depth_frac+delta_frac) < 1 ||
			 delta_dir==-1 && depth_frac < delta_frac ){

			matrix[1][0] = 2*matrix[1][1] - matrix[1][2];
			matrix[2][0] = 2*matrix[2][1] - matrix[2][2];
			numval = 9;
		}
	}

	/* Use the matrix to interpolate the required time. */
	if ( numval==9 ){
		/* Derive travel time for each delta by interpolating over depth */
		for (i=0; i<3; i++){
			t[i] = interpol(depth_frac,matrix[0][i],matrix[1][i],matrix[2][i]);
			d[i] = int_diff(depth_frac,matrix[0][i],matrix[1][i],matrix[2][i]);
		}
		if (diagnostic){
			fprintf(logfp,"ps: t=%f %f %f\n",t[0],t[1],t[2]);
			fprintf(logfp,"ps: d=%f %f %f\n",d[0],d[1],d[2]);
		}

		/* Derive required travel time by interpolating over delta */
		*ttime = 0.1 * interpol(delta_frac,t[0],t[1],t[2]);
		*dtdd = delta_dir * 0.1 * int_diff(delta_frac,t[0],t[1],t[2]);
		*dtdh = depth_dir* 10 * interpol(delta_frac,d[0],d[1],d[2])/MOHO_RADIUS;
	}
	else {
		sprintf(errstr,"del=%f,dep=%f,numval=%d",delta,depth_erad,numval);
		return 2;
	}

	if (diagnostic)
		fprintf(logfp,"ps: ttime=%f dtdd=%f dtdh%f\n",*ttime,*dtdd,*dtdh);

	return 0;
}

/*#DOC  Title:																*/
/*#DOC    fill_matrix ( only called by ps)									*/
/*#DOC  Desc:																*/
/*#DOC    Fill a 3x3 matrix with values from the given table.				*/
/*#DOC  Input Arguments:													*/
/*#DOC    tp        - pointer to structure with travel-time table.			*/
/*#DOC    col1      - first column reuired.									*/
/*#DOC    row1      - first row required.									*/
/*#DOC    depth_dir - whether to go up or down the table.					*/
/*#DOC    delta_dir - whether to go left or right across the table.			*/
/*#DOC    matrix    - 2D array for numbers from table.						*/
/*#DOC  Return:																*/
/*#DOC    number of values found (9 for success).							*/

int fill_matrix (struct tab_rec *tp, int col1, int row1,
				 int depth_dir, int delta_dir, int matrix[3][3])
{
	int row,col,numval;
	int i,j;

	numval = 0;
	for (i=0; i<3; i++){
		row = row1 + i*delta_dir;

		for (j=0; j<3; j++){
			col = col1 + j*depth_dir;

			/* If off the end column use last columns to extrapolate depth */
			if (col>13)
				matrix[j][i] = col*(tp->tab[13][row]) - tp->tab[12][row];
			else
				matrix[j][i] = tp->tab[col][row];

			if (matrix[j][i] != 0)
				numval++;
		}
	}
	return numval;
}

/*#DOC  Title:																*/
/*#DOC    ps_crust															*/
/*#DOC  Desc:																*/
/*#DOC    Calculates P/S ttime, dtdd, and dtdh when source above MOHO.		*/
/*#DOC  Input Arguments:													*/
/*#DOC    ps    - either 'P' or 'S'.										*/
/*#DOC    delta - distance between phase and source in degrees.				*/
/*#DOC    depth - depth of current solution.								*/
/*#DOC    ttime - pointer to variable for travel time.						*/
/*#DOC    dtdh  - pointer to variable for dt/dh.							*/
/*#DOC    dtdd  - pointer to variable for dt/dd.							*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/
/*#DOC  Calls:																*/
/*#DOC    interpol() and int_diff() from this file.							*/

/* Subroutine to calculate travel times when depth < MOHO.					*/
/* Delta will be > 8 or would have been identified as a crustal phase.		*/
/* (assuming using complementary id_pha.)									*/
/* Taken straight from reviser:												*/
/*   ! For method see														*/
/*   !       Arnold,E.P.													*/
/*   !              1965   Revision of the seismological tables				*/
/*   !       PH.D. Thesis Cantab.											*/
/*   !       see  also														*/
/*   !       Arnold,E.P.													*/
/*   !           1967   P and S times in Japan								*/
/*   !       Pubs. of B.C.I.S., Series A   Vol.24, pp.103-135				*/
/*   !       see equation 10 on page 114									*/

int ps_crust(char ps, double delta, double depth,
									double *ttime, double *dtdh, double *dtdd)
{
	double g_slow,b_slow,hfn,ray;
	int times[3];
	int row1,row;
	int delta_dir;
	double delta_frac;
	struct tab_rec* tp;
	int i;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	if (ps == 'P'){
		g_slow = PGSLOW;
		b_slow = PBSLOW;
		tp = &p_tab;
	}
	else if (ps == 'S'){
		g_slow = SGSLOW;
		b_slow = SBSLOW;
		tp = &s_tab;
	}
	else {
		sprintf(errstr,"ps_crust: wrong ps value %c",ps);
		return 1;
	}		

	/* Check that depth is sensible */
	if (depth < 0 || depth > MOHO ){
		sprintf(errstr,"ps_crust: depth out of range: %f",depth);
		return 1;
	}

	/* Get row by using delta */
	if (delta < tp->min_delta || delta > tp->max_delta){
		sprintf(errstr,"ps_crust: delta out of range: %f",delta);
		return 1;
	}
	row1 = delta - tp->min_delta;
	delta_frac = delta - (int)delta;

	/* Interpolate forwards in first half of table and backwards in second */
	/* half of table to avoid the ends. */
	if (delta > tp->switch_dir_delta){
		delta_dir = -1;
		row1++;
		delta_frac = 1 - delta_frac;
	}
	else
		delta_dir = 1;

	if (diagnostic){
		fprintf(logfp,"ps_crust: delta=%f row1=%d ",delta,row1);
		fprintf(logfp,"delta_frac=%f delta_dir=%d\n",delta_frac,delta_dir);
	}

	/* Store the 3 values surrounding the delta of interest. */
	for (i=0; i<3; i++){
		row = row1 + i*delta_dir;

		if (tp->tab[0][row] == 0){
			sprintf(errstr,"no value: del=%d, dep=%f",row1,depth);
			return 1;
		}			
		times[i]=tp->tab[0][row];
	}
	if (diagnostic){
		fprintf(logfp,"ps_crust: t[0]=%d ",times[0]);
		fprintf(logfp,"t[1]=%d t[2]=%d\n",times[1],times[2]);
	}

	/* Calculate time from surface focus. */
	*ttime = 0.1 * interpol(delta_frac,times[0],times[1],times[2]);
	*dtdd  = 0.1 * int_diff(delta_frac,times[0],times[1],times[2])*delta_dir;

	if (diagnostic)
		fprintf(logfp,"ps_crust: surface ttime=%f dtdd=%f\n",*ttime,*dtdd);
	

	/* Correct for actual depth. */
	hfn = (EARTH_RADIUS - depth) * TORAD;
	ray = sqrt((g_slow - *dtdd) * (g_slow + *dtdd));

	if (depth <= CONRAD)
		*ttime -= (ray * depth) / hfn;
	else{
		*ttime -= (ray * CONRAD) / hfn;
		ray = sqrt((b_slow - *dtdd) * (b_slow + *dtdd));
		*ttime -= (ray * (depth - CONRAD)) / hfn;
	}
	*dtdh = -0.57 * ray * 100/MOHO_RADIUS;

	if (diagnostic)
		fprintf(logfp,"ps_crust: ttime=%f dtdh=%f\n",*ttime,*dtdh);

	return 0;
}

/*#DOC  Title:																*/
/*#DOC    ps_short															*/
/*#DOC  Desc:																*/
/*#DOC    Looks up P/S ttime, dtdd, and dtdh when delta < 1.3.				*/
/*#DOC  Input Arguments:													*/
/*#DOC    tp    - pointer to structure with travel-time table.				*/
/*#DOC    delta - distance between phase and source in degrees.				*/
/*#DOC    depth - depth of current solution.								*/
/*#DOC    ttime - pointer to variable for travel time.						*/
/*#DOC    dtdh  - pointer to variable for dt/dh.							*/
/*#DOC    dtdd  - pointer to variable for dt/dd.							*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/
/*#DOC  Calls:																*/
/*#DOC    interpol() and int_diff() from this file.							*/

/* Subroutine to calculate travel times for P and S when delta < 1.3.		*/
/* Depth must be < 0.016 earth radii and will be > MOHO or would have been 	*/
/* identified as a crustal phase (assuming using complementary id_pha.)		*/
/* Calculate time and derivatives using 'Short Epicentral Distances' table.	*/

int ps_short( struct tab_rec* tp, double delta, double depth,
					double* ttime, double* dtdh, double* dtdd)
{
	double depth_frac,delta_frac;
	double depth_erad;
	int times[3];
	int col1,row1,col,row;
	double t[3],d[3];
	int i,j;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* Get column by using depth in earth radii * 500 */
	depth_erad = (depth - MOHO)/(MOHO_RADIUS);
	if (depth_erad < tp->min_depth || depth_erad > tp->max_depth){
		sprintf(errstr,"ps_short:depth out of range: %f",depth);
		return 1;
	}
	col1  = depth_erad*500; 			
	depth_frac = depth_erad*500 - col1;

	/* Get row by using delta * 10 */
	if (delta < tp->min_delta || delta > tp->max_delta){
		sprintf(errstr,"ps_short:delta out of range: %f",delta);
		return 1;
	}
	row1 = delta*10;
	delta_frac = delta*10 - row1;

	if (diagnostic){
		fprintf(logfp,"ps_short: depth=%f col1=%d ",depth_erad,col1);
		fprintf(logfp,"depth_frac=%f\n",depth_frac);
		fprintf(logfp,"ps_short: delta=%f row1=%d ",delta,row1);
		fprintf(logfp,"delta_frac=%f\n",delta_frac);
	}

	for (i=0; i<3; i++){
		row = row1 + i;
		for (j=0; j<3; j++){
			col = col1 + j;
			times[j]=tp->tab[col][row];
		}
		t[i] = interpol(depth_frac,times[0],times[1],times[2]);
		d[i] = int_diff(depth_frac,times[0],times[1],times[2]);
	}
	*ttime = 0.1 * interpol(delta_frac,t[0],t[1],t[2]);
	*dtdd  = int_diff(delta_frac,t[0],t[1],t[2]);
	*dtdh  = 50 * interpol(delta_frac,d[0],d[1],d[2]) / MOHO_RADIUS;

	if (diagnostic)
		fprintf(logfp,"ps_short: tt=%f dtdd=%f dtdh=%f\n",*ttime,*dtdd,*dtdh);

	return 0;
}

/*#DOC  Title:																*/
/*#DOC    ps_g																*/
/*#DOC  Desc:																*/
/*#DOC    Calculates Pg/Sg ttime, dtdd, and dtdh.							*/
/*#DOC  Input Arguments:													*/
/*#DOC    ps    - either 'P' or 'S'.										*/
/*#DOC    delta - distance between phase and source in degrees.				*/
/*#DOC    depth - depth of current solution.								*/
/*#DOC    ttime - pointer to variable for travel time.						*/
/*#DOC    dtdh  - pointer to variable for dt/dh.							*/
/*#DOC    dtdd  - pointer to variable for dt/dd.							*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/

/* Subroutine to calculate travel times for Pg and Sg.						*/
/* Calculate travel time directly using velocities in header file.			*/

int ps_g(char ps, double delta, double depth,
					double *ttime, double *dtdh, double *dtdd)
{
	double g_vel,g_slow;
	double distance;
	double g_ray;

	/* Need velocity in km/s so calculate them from slowness.				*/
	if (ps == 'P'){
		g_slow = PGSLOW;
		g_vel  = PGVEL;
	}
	else if (ps == 'S'){
		g_slow = SGSLOW;
		g_vel  = SGVEL;
	}
	else {
		sprintf(errstr,"ps_g: wrong ps value %c",ps);
		return 1;
	}
	distance = delta * KM_PER_DEG;

	/* Check that depth is sensible */
	if (depth < 0 || depth > CONRAD ){
		sprintf(errstr,"ps_g: depth out of range: %f",depth);
		return 1;
	}

	/* Calculate travel time. */
	g_ray = sqrt(distance*distance + depth*depth);
	*ttime = g_ray/g_vel;

	/* Calculate derivatives dtdh in km. */
	if (*ttime == 0){
		*dtdd = g_slow;
		*dtdh = 0;
	}
	else{
		*dtdd = g_slow*g_slow*delta/(*ttime);
		*dtdh = depth/(*ttime*g_vel*g_vel);
	}
	return 0;
}

/*#DOC  Title:																*/
/*#DOC    ps_star															*/
/*#DOC  Desc:																*/
/*#DOC    Calculates P* / S* ttime, dtdd, and dtdh.							*/
/*#DOC  Input Arguments:													*/
/*#DOC    ps    - either 'P' or 'S'.										*/
/*#DOC    delta - distance between phase and source in degrees.				*/
/*#DOC    depth - depth of current solution.								*/
/*#DOC    ttime - pointer to variable for travel time.						*/
/*#DOC    dtdh  - pointer to variable for dt/dh.							*/
/*#DOC    dtdd  - pointer to variable for dt/dd.							*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/

/* Subroutine to calculate travel times for P* and S*.						*/
/* When source above Conrad calculate travel time directly. 				*/
/* When source below Conrad use converted star subroutine from reviser to	*/
/* iterate for the point where the ray crosses the Conrad and is refracted.	*/
/* Following is Fortran comment from star.f:

c subroutine star for finding travel-times of the direct wave
c with the focus in the middle layer. Uses Newton's method of
c approximation to solve the distance to the refraction point:
c
c      x(n+1)=x(n)-f(x(n))/f'(x(n))
c where f'(x(n))=(f(x(n)+D)-f(x(n)-d))/(2*d)
c   (with acknowlegment to DSIR, New Zealand for a copy of their
c     pstar program)
*/

int ps_star(char ps, double delta, double depth,
									double *ttime, double *dtdh, double *dtdd)
{
	double g_vel,b_vel;
	double distance,above_conrad,below_conrad;
	double conrad_radius,source_radius;
	double frac1,frac2;
	double g_ray,b_ray;
	double prev_crossing,prev_time,incr;
	double crossing,sin_inc,sin_ref,critical,new_dist;
	double cos_inc;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* Need velocities in km/s so calculate them from slownesses.			*/
	if (ps == 'P'){
		g_vel  = PGVEL;
		b_vel  = PBVEL;
	}
	else if (ps == 'S'){
		g_vel  = SGVEL;
		b_vel  = SBVEL;
	}
	else {
		sprintf(errstr,"ps_star: wrong ps value %c",ps);
		return 1;
	}
	distance = delta * KM_PER_DEG;

	/* Check that depth is sensible */
	if (depth < 0 || depth > MOHO ){
		sprintf(errstr,"ps_star: depth out of range: %f",depth);
		return 1;
	}

	if (diagnostic)
		fprintf(logfp,"ps_star: distance=%f,depth=%f\n",distance,depth);

	/* Case when source above Conrad. 	*/
	if (depth < CONRAD ){
		above_conrad = CONRAD - depth;
		critical = asin(g_vel/b_vel);
		*ttime  = (above_conrad+CONRAD)/(g_vel*cos(critical));
		*ttime += (distance - (above_conrad+CONRAD)*tan(critical))/b_vel;

		if (diagnostic)
			fprintf(logfp,"ps_star: depth=%f,tt=%f\n",depth,*ttime);

		/* dt/dh =  (MOHO_RADIUS/vel*vel) * (h/t)		*/
		/* h/t = r.cos(critical)/t = vel.cos(critical)  */
		*dtdh = -cos(critical)/g_vel;
		*dtdd = KM_PER_DEG/b_vel;

		return 0;
	}

	/* Case where source below Conrad.	*/
	below_conrad  = depth - CONRAD;

	/* Special case of station over source. */
	if (distance < 0.01){
		*ttime = below_conrad/b_vel;
		*dtdh  = b_vel;
		*dtdd  = 0;
		return 0;
	}
	conrad_radius = EARTH_RADIUS - CONRAD;
	source_radius = EARTH_RADIUS - depth;

	frac1 = conrad_radius/EARTH_RADIUS;
	frac2 = source_radius/EARTH_RADIUS;

	/* Search for Conrad crossing point that gives right distance. */
	prev_crossing = 0;
	prev_time = 1000000;
	incr = distance/2;
	for(;;){
		crossing = prev_crossing + incr;
		b_ray = sqrt(below_conrad*below_conrad + frac1*frac2*crossing*crossing);
		*ttime   = b_ray/b_vel;
		sin_inc  = frac2*crossing/b_ray;
		sin_ref  = sin_inc*g_vel/b_vel;
		critical = sqrt(1-sin_ref*sin_ref);
		g_ray    = CONRAD/critical;
		*ttime += g_ray/g_vel;

		/* Check to see if close enough yet. */
		new_dist = crossing + g_ray*sin_ref;
		if (fabs(new_dist-distance) < 0.1)
			break;
		if (fabs(new_dist-distance) < 0.5 && fabs(*ttime-prev_time) < 0.01)
			break;

		/* Check to see if gone too far. */
		if (new_dist > distance){
			incr /= 2;
			if (incr < 0.01)
				break;
			continue;
		}
		prev_time = *ttime;
		prev_crossing = crossing;
	}

	/* Calculate derivatives dtdh in km. */
	*dtdd = (sin_inc/b_vel)*KM_PER_DEG;
	cos_inc = 1 - sin_inc*sin_inc;
	if (cos_inc > 0)
		*dtdh = (sqrt(cos_inc)/b_vel);
	else
		*dtdh = 0;

	return 0;
}

/*#DOC  Title:																*/
/*#DOC    pdiff																*/
/*#DOC  Desc:																*/
/*#DOC    Calculates travel time for diffracted P.							*/
/*#DOC  Input Arguments:													*/
/*#DOC    tp   - pointer to structure with travel-time table.				*/
/*#DOC    delta - distance between phase and source in degrees.				*/
/*#DOC    depth - depth of current solution.								*/
/*#DOC    ttime - pointer to variable for travel time.						*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/

/* Calculate diffracted P travel time using the final row of the P			*/
/* table as a starting point and adding 4.4 seconds per degree over that.	*/
/* Taken straight from reviser.												*/
/* Limited use because does not calculate the derivatives.					*/

int pdiff( struct tab_rec* tp, double delta, double depth, double* ttime)
{
	double depth_frac,depth_erad,dtdh;
	int col;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* Only get Pdiff once P has stopped */
	if (delta < PDIFF_MIN_DELTA || delta > 180){
		sprintf(errstr,"pdiff: delta out of range: %f",delta);
		return 1;
	}

	/* Get column by using depth in earth radii * 100 */
	/* One out of step because 1st column is surface, 2nd column is 0.00 */
	depth_erad = (depth - MOHO)/MOHO_RADIUS;
	if (depth_erad < SURFACE || depth_erad > tp->max_depth){
		sprintf(errstr,"pdiff:depth out of range: %f",depth);
		return 1;
	}
	col  = depth_erad*100+1;

	if (col>0)
		depth_frac = depth_erad*100 - (int)(depth_erad*100);
	else if (col==0)
		depth_frac = 1 + depth_erad*MOHO_RADIUS/MOHO;
	else{
		col=0;
		depth_frac = 0;
	}
	if (col==13)
		col=12;

	dtdh= 0.1*(tp->tab[col+1][PDIFF_MIN_DELTA] - tp->tab[col][PDIFF_MIN_DELTA]);
	*ttime = 0.1 * tp->tab[col][PDIFF_MIN_DELTA] + depth_frac*dtdh;
	*ttime = *ttime + PDIFF_SEC_PER_DEG * (delta-PDIFF_MIN_DELTA);

	if (diagnostic){
		fprintf(logfp,"pdiff: delta=%f depth=%f col=%d ",delta,depth,col);
		fprintf(logfp,"depth_frac=%f ttime=%f\n",depth_frac,*ttime);
	}
	return 0;
}

/*#DOC  Title:																*/
/*#DOC    other_phase														*/
/*#DOC  Desc:																*/
/*#DOC    Looks up travel times in JB tables.								*/
/*#DOC  Input Arguments:													*/
/*#DOC    tp   - pointer to structure with surface focus travel-time table. */
/*#DOC    delta - distance between phase and source in degrees.				*/
/*#DOC    depth - depth of current solution.								*/
/*#DOC    ttime - pointer to variable for travel time.						*/
/*#DOC  Return:																*/
/*#DOC    0/3 for success/phase ID wrong.									*/

/* First looks up surface focus table then corrects for depth using depth	*/
/* allowance table.															*/
/* Does not return any derivatives.				 							*/
/* Because these phases not used in solution its not a serious error if		*/
/* delta or depth shows that they are wrongly identified.  Do, however want	*/
/* to remove the ISC phase ID in this case so have special return value.	*/

int other_phase(struct surf_tab_rec* tp,double delta,double depth,double* ttime)
{
	double depth_frac,delta_frac;
	double depth_erad;
	double surf_time,depth_corr;
	double c[4];
	int col,row;
	struct dep_allow_tab_rec* tp_allow;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* First get surface travel time from surface table. */

	/* Get row by using delta */
	if (delta < tp->min_delta || delta > tp->max_delta){
		sprintf(errstr,"other_phase: delta out of range: %f",delta);
		return 3;
	}
	row = delta - tp->min_delta;
	delta_frac = delta - (int)delta;

	surf_time  = 0.1 * tp->tab[row];
	surf_time += 0.1 * (tp->tab[row+1]-tp->tab[row]) * delta_frac;

	if (diagnostic){
		fprintf(logfp,"other_phase: delta=%f delta_frac=%f ",delta,delta_frac);
		fprintf(logfp,"surf_time=%f\n",surf_time);
	}

	/* Then get depth correction. */
	tp_allow = tp->dep_allow_tp;

	/* Rows are unevenly spaced - delta of each row given in row_label array. */
	for (row=0; row < tp_allow->numrows; row++)
		if (delta < tp_allow->row_label[row+1])
			break;

	delta_frac = (delta - tp_allow->row_label[row]) / 
	             (tp_allow->row_label[row+1] - tp_allow->row_label[row]);

	if (diagnostic)
		fprintf(logfp,"other_phase: row=%d delta_frac=%f\n",row,delta_frac);

	/* Get column by using depth in earth radii * 100 */
	/* Reject if exactly at end - won't be able to extrapolate. */
	depth_erad = (depth - MOHO)/MOHO_RADIUS;
	if (depth_erad < SURFACE || depth_erad >= tp_allow->max_depth){
		sprintf(errstr,"other_phase: depth out of range: %f",depth);
		return 3;
	}

	/* Extrapolate for depths above Moho.*/
	if (depth_erad <= 0){
		col = -1;
		depth_frac = 1 - depth_erad/SURFACE;
		c[0] = 0;
		c[1] = 0;
	}
	/* Otherwise interpolate. */
	else{
		col  = depth_erad*100; 			
		depth_frac = depth_erad*100 - col;
		c[0] = 0.1 * tp_allow->tab[row][col];
		c[1] = 0.1 * tp_allow->tab[row+1][col];
	}
	c[2] = 0.1 * tp_allow->tab[row][col+1];
	c[3] = 0.1 * tp_allow->tab[row+1][col+1];

	if (diagnostic){
		fprintf(logfp,"other_phase: depth=%f depth_erad=%f ",depth,depth_erad);
		fprintf(logfp,"depth_frac=%f ",depth_frac);
		fprintf(logfp,"c= %f %f %f %f\n",c[0],c[1],c[2],c[3]);
	}

	depth_corr  = c[0] + delta_frac*(c[1]-c[0]);
	depth_corr += depth_frac*(c[2]-c[0]);
	depth_corr += depth_frac*delta_frac*(c[0]-c[1]+c[3]-c[2]);

	if (diagnostic)
		fprintf(logfp,"other_phase:depth_corr=%f\n",depth_corr);

	*ttime = surf_time - depth_corr;
	return 0;
}

/*#DOC  Title:																*/
/*#DOC    jb_pP_P															*/
/*#DOC  Desc:																*/
/*#DOC    Looks up pP - P time and dtdh in JB table.						*/
/*#DOC  Input Arguments:													*/
/*#DOC    delta - distance between phase and source in degrees.				*/
/*#DOC    depth - depth of current solution.								*/
/*#DOC    timediff - pointer to variable for difference in travel times.	*/
/*#DOC    dtdh  - pointer to variable for dt/dh.							*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/
/*#DOC  Calls:																*/
/*#DOC    interpol() and int_diff() from this file.							*/

/* This is the only function here not called by read_jb(). Rather it is		*/
/* called from read_pP_P(), which provides a table independent wrapper for	*/
/* calculating P-pP times.													*/

int jb_pP_P( double delta, double depth, double* timediff, double* dtdh)
{
	double depth_erad,moho;
	double depth_frac,delta_frac;
	int times[3];
	int col1,row1,col,row;
	double t[3],d[3];
	int delta_dir;
	int i,j;
	struct tab_rec* tp;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	tp = &pP_P_tab;

	/* Get column by using depth in earth radii * 100 */
	/* One out of step because 1st column is surface, 2nd column is 0.00 */
	depth_erad = (depth - MOHO)/MOHO_RADIUS;
	if (depth_erad < tp->min_depth || depth_erad > tp->max_depth){
		sprintf(errstr,"jb_pP_P: depth out of range: %f",depth);
		return 1;
	}
	col1  = depth_erad*100 + 1;
	depth_frac = depth_erad*100 - (int)(depth_erad*100);

	/* Get row by using delta / 5 */
	if (delta < tp->min_delta || delta > tp->max_delta){
		sprintf(errstr,"jb_pP_P: delta out of range: %f",delta);
		return 1;
	}
	row1 = delta*0.2;
	delta_frac = delta*0.2 - row1;

	if (diagnostic){
		fprintf(logfp,"jb_pP_P: depth=%f depth_erad=%f ",depth,depth_erad);
		fprintf(logfp,"col1=%d depth_frac=%f ",col1,depth_frac);
		fprintf(logfp,"delta=%f,row1=%d,frac=%f\n",delta,row1,delta_frac);
	}

	/* Depth above MOHO - extrapolate off edge of table. */
	if (col1 == 0){
		row = row1 + 1;

		t[0]=tp->tab[1][row];
		t[1]=tp->tab[1][row+1];

		if (t[0]==0 || t[1]==0)
			return 0;

		moho = 100*MOHO/MOHO_RADIUS;

		*timediff = 0.1 * (1 + depth_frac/moho)*(delta_frac*(t[1]-t[0]) + t[0]);

		/* dtdh in km. */
		*dtdh     = 0.1 * (t[0]/MOHO);
	}
	/* Depth below MOHO. */
	else {

		/* Interpolate forwards in first half of table and backwards */
		/* in second half of table to avoid the ends. */
		if (delta > tp->switch_dir_delta){
			delta_dir = -1;
			row1++;
			delta_frac = 1 - delta_frac;
		}
		else
			delta_dir = 1;

		/* Store the 3 values surrounding the delta of interest. */
		for (i=0; i<3; i++){
			row = row1 + i*delta_dir;
			for (j=0; j<3; j++){
				col = col1 + j;

				/* If on the table. */
				if (col<14){
					times[j] = tp->tab[col][row];
					if (times[j]==0)
						return 0;
				}
				/* If off the bottom of the table. */
				else{
					times[j]  = tp->tab[13][row];
					times[j] += (col-13)*(tp->tab[14][row]-tp->tab[13][row]);
				}
			}

			if (diagnostic)
				fprintf(logfp,"t[%d]=%d %d %d\n",i,times[0],times[1],times[2]);

			t[i] = interpol(depth_frac,times[0],times[1],times[2]);
			d[i] = int_diff(depth_frac,times[0],times[1],times[2]);
		}
		*timediff = 0.1 * interpol(delta_frac,t[0],t[1],t[2]);

		/* Convert dtdh from e.r.*100 to km. 0.1*100=10 */
		*dtdh = 10*interpol(delta_frac,d[0],d[1],d[2])/MOHO_RADIUS;
	}
	return 0;
}

/*#DOC  Title:																*/
/*#DOC    interpol															*/
/*#DOC  Desc:																*/
/*#DOC    Newton Gregory interpolation from three points.					*/
/*#DOC  Input Arguments:													*/
/*#DOC    frac  - fraction of table interval.								*/
/*#DOC    v1, v2, v3 - table values to either side.							*/
/*#DOC  Return:																*/
/*#DOC    Interpolated value.												*/

double interpol ( double frac, double v1, double v2, double v3 )
{
	double val;

	val  = v1 * ( 1 - 0.5 * frac * (3-frac) );
	val += v2 * frac * (2-frac);
	val += v3 * 0.5 * frac * (frac-1);

	return val;
}

/*#DOC  Title:																*/
/*#DOC    int_diff															*/
/*#DOC  Desc:																*/
/*#DOC    Newton Gregory difference interpolation from three points.		*/
/*#DOC  Input Arguments:													*/
/*#DOC    frac  - fraction of table interval.								*/
/*#DOC    v1, v2, v3 - table values to either side.							*/
/*#DOC  Return:																*/
/*#DOC    Interpolated difference.											*/

double int_diff ( double frac, double v1, double v2, double v3 )
{
	double val;

	val  = v1 * (frac-1.5);
	val += v3 * (frac-0.5);
	val -= v2 * 2 * (frac-1);

	return val;
}

