#include "location.h"

/*#DOC  Title:																*/
/*#DOC    solve																*/
/*#DOC  Desc:																*/
/*#DOC    Solves equations and updates solution structure.					*/
/*#DOC  Input Arguments:													*/
/*#DOC    sp - pointer to structure containing solution.					*/
/*#DOC    p  - array of pointers to structures containing phases.			*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/

/* Does not calculate errors - the covariance matrix is stored so that	*/
/* another function can do this.										*/

int solve (struct sol_rec *sp, struct pha_rec p[])
{
	extern double max_err_factor;		/* From config file */

	double *simula[5];		/* Maximum of 4 unkowns - don't use index 0. */
	double *simulb;
	double c[5][5];
	double d[5];
	double e[5];
	double f[10][5];
	double ermax[5];
	double y;
	double sqrt_weight;
	int dir;
	int nunk,num,npp;
	int i,j,k;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* Local variables with nice short names for use in for statements etc. */
	nunk = sp->number_of_unknowns;
	num  = sp->numphas;
	npp  = nunk*2 +1;

	/* If everything fixed than nothing to do. */
	if (nunk == 0){
		return 0;
	}
	else if (nunk==2 || nunk > 4){
		sprintf(errstr,"solve: number of unknowns wrong %d",nunk);
		return 1;
	}

	/* Assign memory for arrays of simultaneous equation coefficients. */
	if ((simulb = (double*) calloc (num, sizeof(double))) == NULL){
		sprintf(errstr,"solve: Memory problem, simulb.");
		return 1;
	}
	for (i=0;i<=5;i++){
		if ((simula[i] = (double*) calloc (num, sizeof(double))) == NULL){
			sprintf(errstr,"solve: Memory problem, simula[%d].",i);
			free(simulb);
			for(j=0;j<i;j++)
				free(simula[j]);
			return 1;
		}
	}

	/* Fill in equation for each phase. */
	/* And apply weights. */
	for (i=0; i<num; i++){
		if (p[i].weight_factor==0)
			continue;

		simula[1][i] = 1;
		simulb[i] = p[i].resid;

		if (nunk > 2){
			simula[2][i] = -p[i].dtdd * cos(p[i].esaz*TORAD);
			simula[3][i] = -p[i].dtdd * sin(p[i].esaz*TORAD);
		}
		if (nunk> 3)
			simula[4][i] = p[i].dtdh;

		sqrt_weight = sqrt(p[i].weight);
		simulb[i] *= sqrt_weight;

		if (diagnostic){
			fprintf(logfp,"solve: %s %s ",p[i].sta,p[i].phase);
			fprintf(logfp,"simulb=%f\n",simulb[i]);
		}
		for (j=1; j<=nunk; j++)
			simula[j][i] *= sqrt_weight;
	}

	/* Fill matrix C and vector D */
	for (i=1; i<=nunk; i++){
		for (j=1; j<=nunk; j++){
			c[j][i] = 0;
			for (k=0; k<num; k++)
				if (p[k].weight_factor > 0)
					c[j][i] += simula[i][k] * simula[j][k];
		}
		d[i] = 0;
		for (k=0; k<num; k++)
				if (p[k].weight_factor > 0){
					d[i] += simula[i][k] * simulb[k];
				}
	}

	/* Free memory used for simultaneous equation coefficients. */
	free(simulb);
	for (i=0;i<=5;i++)
		free(simula[i]);

	if (diagnostic){
		fprintf(logfp,"solve:c1=%f %f %f %f\n",c[1][1],c[2][1],c[3][1],c[4][1]);
		fprintf(logfp,"      c2=%f %f %f %f\n",c[1][2],c[2][2],c[3][2],c[4][2]);
		fprintf(logfp,"      c3=%f %f %f %f\n",c[1][3],c[2][3],c[3][3],c[4][3]);
		fprintf(logfp,"      c4=%f %f %f %f\n",c[1][4],c[2][4],c[3][4],c[4][4]);
		fprintf(logfp,"       d=%f %f %f %f\n\n",d[1],d[2],d[3],d[4]);
	}

	/* Augment C with D and unitary matrix to get F. */
	for (i=1; i<=nunk; i++){

		for (j=1; j<=nunk; j++)
			f[j][i] = c[j][i];

		f[nunk+1][i] = d[i];

		for (j=nunk+2; j<=npp; j++)
			f[j][i] = 0;
		
		f[nunk+1+i][i] = 1;
	}

	/* Set minimum values for diagonal elements. */
	/* Depth minimum adjusted to match dtdh in s/km. */
	ermax[1] = nunk + 0.5 * max_err_factor;
	ermax[2] = nunk * max_err_factor;
	ermax[3] = nunk * max_err_factor;
	ermax[4] = 0.0001 * max_err_factor;

	/* Triangulate F */
	for (i=1; i<=nunk; i++){

		/* Check for diagonals getting too small. */
		if (f[i][i] < ermax[i]){
			fprintf(logfp,"SOLUTION INDETERMINATE, row %d\n",i);
			if (diagnostic)
				fprintf(logfp,"ermax=%f value=%f\n",ermax[i],f[i][i]);
			strcpy(errstr,"");
			return 1;
		}

		y = 1/f[i][i];
		for (j=i; j<=npp; j++)
			f[j][i] = f[j][i]*y;

		if (i<nunk){
			for(j=i+1; j<=nunk; j++){
				y = f[i][j];
				for (k=i; k<=npp; k++)
					f[k][j] -= y * f[k][i];
			}
		}
	}

	/* Back-substitute for solution. */
	e[nunk] = f[nunk+1][nunk];
	if (nunk>1){
		for (i=nunk-1; i>0; i--){
			e[i] = f[nunk+1][i];
			for (j=i+1; j<=nunk; j++)
				e[i] -= e[j]*f[j][i];
		}
		/* Back-substitute for standard errors. */
		for (i=2*nunk; i>nunk+1; i--)
			for (j=nunk-1; j>0; j--)
				for (k=j+1; k<=nunk; k++)
					f[i][j] -= f[k][j]*f[i][k];
	}

	if (diagnostic)
		fprintf(logfp,"solve: e=%f %f %f %f\n\n",e[1],e[2],e[3],e[4]);

	/* Store covariance matrix in solution structure. */
	for (i=1; i<=nunk; i++)
		for (j=1; j<=nunk; j++)
			sp->covar[j][i] = f[nunk+1+j][i];

	/* Update solution. */
	sp->time += e[1];

	if (nunk>1){
		sp->lat += e[2];
		if (fabs(sp->lat) >= 90){
			sp->lat = sp->lat/fabs(sp->lat) * 180 - sp->lat;
			sp->lon = sp->lon/fabs(sp->lon) * 180 - sp->lon;
			dir = -1;
		}
		else
			dir = 1;

		sp->lon += dir * e[3]/cos(sp->lat*TORAD);
		if (fabs(sp->lon) >= 180)
			sp->lon -= sp->lon/fabs(sp->lon) * 360;

		calc_dircos(sp->lat,sp->lon,sp->dircos);

		if (nunk==4){
			sp->depth += e[4];

			/* Check for negative depth. */
			if (sp->depth < 0){
				fprintf(logfp,"NEGATIVE DEPTH\n");
				strcpy(errstr,"");
				return 1;
			}
		}
	}
	return 0;
}
