#include "location.h"

#define XSQ_LIM 70				/* Limit on exponent part of weight. */
#define MIN_TOTAL_WEIGHT 1.9	/* Minimum summed weight to do calculation. */

/*#DOC  Title:																*/
/*#DOC    solve_depdp														*/
/*#DOC  Desc:																*/
/*#DOC    Fills 'depdp' and 'depdp_error' in solution if enough pP phases.	*/
/*#DOC  Input Arguments:													*/
/*#DOC    sp - pointer to structure containing current solution.			*/
/*#DOC    p  - array of phase structures.									*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/
/*#DOC  Calls:																*/
/*#DOC	  read_pP_P() from read_pP_P.c										*/

/* Normally only called once per event but if called a second time will use	*/
/* previous weights to get alpha for this calculation.						*/

int solve_depdp (struct sol_rec *sp, struct pha_rec p[])
{
	extern double init_max_resid;		/* From config file */

	double alpha,total_weight;
	double xsq,sum1,sum2;
	double pP_P_ob;
	int i,num_pP;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

	/* Calculate preliminary weights and alpha.		*/
	/* Ignore residuals that are too big.			*/
	if (sp->depdp == NULLVAL){

		/* Starting alpha when no weights yet.	*/
		alpha = total_weight = 0;
		for (i=0; i<sp->numphas; i++){
			if (p[i].pP_resid > init_max_resid || p[i].pP_resid == NULLVAL)
				continue;

			alpha += p[i].pP_resid*p[i].weight_factor;
			total_weight++;
		}
		if (total_weight == 0)
			return 0;
		alpha /= total_weight;

		/* Preliminary weights to get alpha for final weights.	*/
		for (i=0; i<sp->numphas; i++){
			if (p[i].pP_resid == NULLVAL)
				continue;

			xsq = 0.125*(p[i].pP_resid - alpha)*(p[i].pP_resid - alpha);
			if (xsq > XSQ_LIM)
				continue;
			p[i].pP_weight = 1 / (1 + 0.1*exp(xsq));
		}
	}

	/* Calculate alpha. */
	alpha = total_weight = 0;
	for (i=0; i<sp->numphas; i++){
		if (p[i].pP_resid == NULLVAL)
			continue;
		alpha += p[i].pP_resid*p[i].pP_weight*p[i].weight_factor;
		total_weight += p[i].pP_weight*p[i].weight_factor;
	}
	/* Nullify depth phase depth in case routine returns early. */
	sp->depdp = NULLVAL;

	/* If no pP phases then can't calculate alpha - or depdp. */
	if (total_weight == 0)
		return 0;

	alpha /= total_weight;

	/* Calculate new weights and solve. */
	sum1 = sum2 = total_weight = num_pP = 0;
	for (i=0; i<sp->numphas; i++){
		if (p[i].pP_resid == NULLVAL || p[i].weight_factor == 0)
			continue;

		xsq = 0.125*(p[i].pP_resid - alpha)*(p[i].pP_resid - alpha);
		if (xsq > XSQ_LIM)
			continue;
		p[i].pP_weight = 1 / (1 + 0.1*exp(xsq));
		sum1 += p[i].weight_factor*p[i].pP_weight * p[i].pP_resid* p[i].pP_dtdh;
		sum2 += p[i].weight_factor*p[i].pP_weight * p[i].pP_dtdh * p[i].pP_dtdh;
		total_weight += p[i].weight_factor*p[i].pP_weight;
		num_pP++;
	}

	if (diagnostic){
		fprintf(logfp,"solve_depdp:sum1=%f,sum2=%f ",sum1,sum2);
		fprintf(logfp,"total_weight=%f\n",total_weight);
	}

	if ( sum2 == 0 || total_weight < MIN_TOTAL_WEIGHT)
		return 0;

	sp->depdp = sp->depth + (sum1/sum2);

	if (sp->depdp < 0){
		fprintf(logfp,"solve_depdp: NEGATIVE DEPTH\n");
		sp->depdp = NULLVAL;
		strcpy(errstr,"");
		return 1;
	}

	/* Calculate new residuals and the error in depdp. */
	sum1=0;
	for (i=0; i<sp->numphas; i++){
		if (p[i].pP_resid == NULLVAL)
			continue;

		/* Calculate backwards from residual for observed time difference. */
		pP_P_ob = p[i].pP_resid + p[i].pP_P_time;

		/* Get pP-P time from tables for new depth. */
		if ( read_pP_P(sp->depdp, &p[i]) ){
			handle_error();
			continue;
		}
		/* Calulate new residual in time difference. */
		p[i].pP_resid = pP_P_ob - p[i].pP_P_time;

		/* Sum the squares of residuals to get error. */
		sum1 += p[i].pP_weight*p[i].pP_resid*p[i].pP_resid;
	}
	sp->depdp_error  = sqrt(sum1/(sum2*total_weight));

	if (diagnostic){
		fprintf(logfp,"solve_depdp: depdp= %.2f +/- ",sp->depdp);
		fprintf(logfp,"%.2f using %d phases\n",sp->depdp_error,num_pP);
	}
	return 0;
}
