#include "location.h"
#include "isf_bul.h"

/*#DOC  Title:																*/
/*#DOC    write_isf_event													*/
/*#DOC  Desc:																*/
/*#DOC    Writes solution to ISF bulletin format file.						*/
/*#DOC  Input Arguments:													*/
/*#DOC    outfile  - file pointer to write to.								*/
/*#DOC    ep  - pointer to structure containing event information.			*/
/*#DOC    sp  - pointer to structure containing current solution.			*/
/*#DOC    p[] - array of structures containing phases.						*/
/*#DOC    h[] - array of structures containing hypocentres.					*/
/*#DOC  Return:																*/
/*#DOC    0/1 for success/failure.											*/
/*#DOC  Uses:																*/
/*#DOC	  split_time() from utils.c.										*/
/*#DOC	  functions from isf_writ.c.										*/

int write_isf_event(FILE *outfile, struct event_rec *ep, struct sol_rec *sp, 
										struct hyp_rec h[], struct pha_rec p[])
{
	extern int repid;				/* From config file */
	extern char out_agency[];		/* From config file */

	int yyyy,mm,dd,hh,mi,ss,msec;
	char antype,loctype,epifix,depfix,timfix;
	double mag,magerr;
	char magind;
	char timedef,azimdef,slowdef;
	char picktype,detchar,sp_fm;
	struct pha_rec temp_pha;
	double azimres,slowres,snr;
	double amp,per;
	char **origin_param, **origin_value,**origin_error;
	char *magtype,*region;
	char *evid, *origid, *arrid;
	int i,j;

	/* Switch debug messages from this function on/off (1/0). */
	int diagnostic = 0;

    region  = (char *)malloc(ISF_REGION_LEN+1);
    evid    = (char *)malloc(ISF_EVID_LEN+1);
    origid  = (char *)malloc(ISF_ORIGID_LEN+1);
    magtype = (char *)malloc(ISF_MAGTYPE_LEN+1);
    arrid   = (char *)malloc(ISF_ARRID_LEN+1);

    origin_param = (char **)malloc(ISF_NUM_PARAM);
    origin_value = (char **)malloc(ISF_NUM_PARAM);
    origin_error = (char **)malloc(ISF_NUM_PARAM);

    origin_param[0] = (char *)malloc(VALLEN);
    origin_value[0] = (char *)malloc(VALLEN);
    origin_error[0] = (char *)malloc(VALLEN);

	/* Don't want to write data_type line every event. */
	strcpy(isf_prev_line_type,"phase");

	/* Write event header. */
	sprintf(evid,"%d",ep->evid);
	strcpy(region,"");
	if (write_event_id(outfile,evid,region)){
		sprintf(errstr,"write_event_id: %s",isf_bulletin_error);
		handle_error();
	}

	/* Write origin header. */
	if (write_origin_head(outfile)){
		sprintf(errstr,"write_origin_head: %s",isf_bulletin_error);
		handle_error();
	}

	/* Write new solution. */

	split_time(sp->time,&yyyy,&mm,&dd,&hh,&mi,&ss,&msec);

	/* Set values of fixed flags. */
	depfix = epifix = timfix = ' ';
	if (sp->number_of_unknowns < 4)
		depfix = 'f';
	if (sp->number_of_unknowns < 3)
		epifix = 'f';
	if (sp->number_of_unknowns < 1)
		timfix = 'f';

	if (sp->sdobs == NULLVAL)   	sp->sdobs = ISF_NULL;
	if (sp->majax == NULLVAL)   	sp->majax = ISF_NULL;
	if (sp->minax == NULLVAL)   	sp->minax = ISF_NULL;
	if (sp->theta == NULLVAL)   	sp->theta = ISF_NULL;
	if (sp->ndef == NULLVAL)   		sp->ndef = ISF_NULL;
	if (sp->ndefsta == NULLVAL)   	sp->ndefsta = ISF_NULL;
	if (sp->azimgap == NULLVAL)   	sp->azimgap = ISF_NULL;
	if (sp->mindist == NULLVAL) 	sp->mindist = ISF_NULL;
	if (sp->maxdist == NULLVAL) 	sp->maxdist = ISF_NULL;
	if (sp->error[1] == NULLVAL)	sp->error[1] = ISF_NULL;
	if (sp->error[4] == NULLVAL)	sp->error[4] = ISF_NULL;

	antype  = ' ';
	loctype = ' ';

	sprintf(origid,"%d",sp->hypid);

	if (write_origin( outfile,yyyy,mm,dd,hh,mi,ss,msec,timfix,sp->error[1],
		sp->sdobs,sp->lat,sp->lon,epifix,sp->majax,sp->minax,sp->theta,
		sp->depth,depfix,sp->error[4],sp->ndef,sp->ndefsta,
		sp->azimgap,sp->mindist,sp->maxdist,antype,loctype,ep->etype,
		out_agency,origid)){

		sprintf(errstr,"write_origin: %s",isf_bulletin_error);
		handle_error();
	}

	/* Put in depth-phase depth as an origin parameter comment. */
	if (sp->depdp != NULLVAL){
		origin_param[0] = "pP_DEPTH";
		sprintf(origin_value[0],"%.2f",sp->depdp);
		sprintf(origin_error[0],"%.2f",sp->depdp_error);

		if (write_origin_param(outfile,origin_param,origin_value,
													origin_error,1)){

			sprintf(errstr,"write_origin_param: %s",isf_bulletin_error);
			handle_error();
		}
	}

	/* Write other hypocentres. */
	for (i=0; i<ep->numhyps; i++){
		split_time(h[i].time,&yyyy,&mm,&dd,&hh,&mi,&ss,&msec);

		depfix = epifix = timfix = ' ';
		if (h[i].depfix)
			depfix = 'f';
		if (h[i].epifix)
			epifix = 'f';
		if (h[i].timfix)
			timfix = 'f';

		sprintf(origid,"%d",h[i].hypid);

		if (h[i].stime == NULLVAL)   h[i].stime   = ISF_NULL;
		if (h[i].sdobs == NULLVAL)   h[i].sdobs   = ISF_NULL;
		if (h[i].majax == NULLVAL)   h[i].majax   = ISF_NULL;
		if (h[i].minax == NULLVAL)   h[i].minax   = ISF_NULL;
		if (h[i].theta == NULLVAL)   h[i].theta   = ISF_NULL;
		if (h[i].depth == NULLVAL)   h[i].depth   = ISF_NULL;
		if (h[i].sdepth == NULLVAL)  h[i].sdepth  = ISF_NULL;
		if (h[i].ndef == NULLVAL)    h[i].ndef    = ISF_NULL;
		if (h[i].ndefsta == NULLVAL) h[i].ndefsta = ISF_NULL;
		if (h[i].azimgap == NULLVAL) h[i].azimgap = ISF_NULL;
		if (h[i].mindist == NULLVAL) h[i].mindist = ISF_NULL;
		if (h[i].maxdist == NULLVAL) h[i].maxdist = ISF_NULL;

		if (write_origin( outfile,yyyy,mm,dd,hh,mi,ss,msec,timfix,h[i].stime,
			h[i].sdobs,h[i].lat,h[i].lon,epifix,h[i].majax,h[i].minax,
			h[i].theta,h[i].depth,depfix,h[i].sdepth,h[i].ndef,h[i].ndefsta,
			h[i].azimgap,h[i].mindist,h[i].maxdist,antype,loctype,h[i].etype,
			h[i].agency,origid)){

			sprintf(errstr,"write_origin: %s",isf_bulletin_error);
			handle_error();
		}
	}

	/* Magnitude block. */
	if (sp->bodymag || sp->surfmag){
		if (write_netmag_head(outfile)){
			sprintf(errstr,"write_netmag_head: %s",isf_bulletin_error);
			handle_error();
		}
		sprintf(origid,"%d",sp->hypid);
		magind  = ' ';
		magerr = ISF_NULL;
	}
	if (sp->bodymag)
		if (write_netmag(outfile,"mb",magind,sp->bodymag,magerr,sp->nsta_mb,
			out_agency,origid)){

			sprintf(errstr,"write_netmag mb: %s",isf_bulletin_error);
			handle_error();
		}
	if (sp->surfmag)
		if (write_netmag(outfile,"MS",magind,sp->surfmag,magerr,sp->nsta_ms,
			out_agency,origid)){

			sprintf(errstr,"write_netmag MS: %s",isf_bulletin_error);
			handle_error();
		}

	/* Phase block. */
	if (write_phase_head(outfile)){
		sprintf(errstr,"write_phase_head: %s",isf_bulletin_error);
		handle_error();
	}

	/* Sort phase structures by time then by sta/delta. */
	for (i=0; i < ep->numphas; i++)
		for (j=i-1; j > -1; j--)
			if ((p[j].time > p[j+1].time && p[j+1].time != NULLVAL)
			|| p[j].time == NULLVAL){
				temp_pha = p[j];
				p[j]     = p[j+1];
				p[j+1]   = temp_pha;
			}

	for (i=0; i < ep->numphas; i++)
		for (j=i-1; j > -1; j--)
			if (strcmp(p[j].sta,p[j+1].sta)>0){
				temp_pha = p[j];
				p[j]     = p[j+1];
				p[j+1]   = temp_pha;
			}

	for (i=0; i < ep->numphas; i++)
		for (j=i-1; j > -1; j--)
			if (p[j].delta > p[j+1].delta){
				temp_pha = p[j];
				p[j]     = p[j+1];
				p[j+1]   = temp_pha;
			}

	/* Fields that will be the same for all phases. */
	azimdef = '_';
	slowdef = '_';
	azimres = ISF_NULL;
	slowres = ISF_NULL;
	snr     = ISF_NULL;
	picktype = '_';
	magind  = ' ';

	for (i=0; i<ep->numphas; i++){

		/* Exclude purged readings. */
		/* Don't split readings by excluding single purged phases. */
		if (p[i].purged == 2)
			continue;

		if (split_time(p[i].time,&yyyy,&mm,&dd,&hh,&mi,&ss,&msec)){
			hh = ISF_NULL;
			mi = ISF_NULL;
			ss = ISF_NULL;
			msec = ISF_NULL;
		}

		if (p[i].weight_factor > 0 && p[i].resid != NULLVAL){
			timedef = 'T';
		}
		else{
			timedef = '_';
		}

		/* Ensure that values will not be rejeted by isf_writ functions. */
		if (p[i].azim == NULLVAL)    p[i].azim    = ISF_NULL;
		if (p[i].slow == NULLVAL)    p[i].slow    = ISF_NULL;
		if (p[i].resid == NULLVAL)   p[i].resid   = ISF_NULL;
		else if (p[i].resid >  9999) p[i].resid   = 9999;
		else if (p[i].resid < -9999) p[i].resid   =-9999;
    	if (p[i].sp_fm=='c' || p[i].sp_fm=='d' || p[i].sp_fm=='_')
			sp_fm = p[i].sp_fm;
		else
			sp_fm = '_';

		/* Only space for one amplitude per row - add rest later */
		if (p[i].numamps > 0 && p[i].a[0].amp != NULLVAL){
			amp = p[i].a[0].amp;
			per = p[i].a[0].per;

			if (amp == NULLVAL) amp = ISF_NULL;
			if (per == NULLVAL) per = ISF_NULL;
		}
		else{
			amp = ISF_NULL;
			per = ISF_NULL;
		}

		/* 'Qual' field made up of three flags. Picktype always '_'. */
		if (p[i].sp_fm == ' ')
			p[i].sp_fm = '_';
		else{
			p[i].sp_fm = tolower(p[i].sp_fm);

			if (p[i].sp_fm == 'u' || p[i].sp_fm == '+')
				p[i].sp_fm = 'c';
			else if (p[i].sp_fm == '-')
				p[i].sp_fm = 'd';
		}
		if (p[i].detchar =='e' || p[i].detchar =='i' || p[i].detchar =='q')
			detchar = p[i].detchar;
		else
			detchar = '_';

		/* Station magnitude - only one. */
		if (p[i].bodymag){
			mag = p[i].bodymag;
			strcpy(magtype,"mb");
		}
		else if (p[i].surfmag){
			mag = p[i].surfmag;
			strcpy(magtype,"MS");
		}
		else{
			mag = ISF_NULL;
			strcpy(magtype,"");
		}

		/* Arrival ID corresponds to database phid. */
		sprintf(arrid,"%d",p[i].phid);

		if (write_phase(outfile,p[i].sta,p[i].delta,p[i].esaz,p[i].phase,
			hh,mi,ss,msec,p[i].resid,p[i].azim,azimres,p[i].slow,slowres,
			timedef,azimdef,slowdef,snr,amp,per,picktype,sp_fm,detchar,
			magtype,magind,mag,arrid)){

			sprintf(errstr,"write_phase:%s: %s",arrid,isf_bulletin_error);
			handle_error();
		}

		/* Other amplitudes from this phase have everything else the same.	*/
		/* Set mags to null as station MS will be an average.				*/
		for (j=1; j<p[i].numamps; j++){
			amp = p[i].a[j].amp;
			per = p[i].a[j].per;

			if (amp == NULLVAL) continue;
			if (per == NULLVAL) continue;

			mag = ISF_NULL;
			strcpy(magtype,"");

			strcpy(arrid,"amp_only");

			if (write_phase(outfile,p[i].sta,p[i].delta,p[i].esaz,p[i].phase,
				hh,mi,ss,msec,p[i].resid,p[i].azim,azimres,p[i].slow,slowres,
				timedef,azimdef,slowdef,snr,amp,per,picktype,sp_fm,detchar,
				magtype,magind,mag,arrid)){

				sprintf(errstr,"write_phase:%s: %s",arrid,isf_bulletin_error);
				handle_error();
			}
		}

	}

    free(origin_param[0]);
    free(origin_value[0]);
    free(origin_error[0]);

    free(origin_param);
    free(origin_value);
    free(origin_error);
	free(region);
	free(evid);
	free(origid);
	free(magtype);
	free(arrid);

	return 0;
}
